@@ -42,6 +42,7 @@
 #if defined(EMBEDDED_SSL) || defined(WOLFSSL_DHX)
 #if defined(WOLFSSL_DHX)
 #include <wolfssl/options.h>
+#include <wolfssl/wolfcrypt/settings.h>
 #endif
 #include <wolfssl/openssl/des.h>
 #elif defined(OPENSSL_DHX)
@@ -66,6 +67,10 @@
 #define HEXPASSWDLEN 16
 #define PASSWDLEN 8
 
+#ifndef DES_KEY_SZ
+#define DES_KEY_SZ (sizeof(DES_cblock))
+#endif
+
 static char buf[MAXPATHLEN + 1];
 
 /* if newpwd is null, convert buf from hex to binary. if newpwd isn't
