@@ -10,43 +10,48 @@
 #include "config.h"
 #endif /* HAVE_CONFIG_H */
 
+#include <arpa/inet.h>
+#include <errno.h>
+#include <netinet/in.h>
+#include <netinet/tcp.h>
+#include <setjmp.h>
+#include <signal.h>
+#include <stdbool.h>
 #include <stdio.h>
 #include <stdlib.h>
-#include <signal.h>
 #include <string.h>
-#include <errno.h>
-#ifdef HAVE_UNISTD_H
-#include <unistd.h>
-#endif /* HAVE_UNISTD_H */
 #include <sys/socket.h>
-#include <sys/time.h>
+
 #ifdef HAVE_SYS_STAT_H
 #include <sys/stat.h>
 #endif /* HAVE_SYS_STAT_H */
-#include <netinet/in.h>
-#include <netinet/tcp.h>
-#include <arpa/inet.h>
-#include <setjmp.h>
-#include <time.h>
+
+#include <sys/time.h>
 #include <sys/types.h>
 #include <sys/wait.h>
+#include <time.h>
+#include <unistd.h>
 
-#include <atalk/logger.h>
-#include <atalk/dsi.h>
 #include <atalk/compat.h>
-#include <atalk/util.h>
-#include <atalk/uuid.h>
-#include <atalk/paths.h>
-#include <atalk/server_ipc.h>
+#include <atalk/dsi.h>
 #include <atalk/fce_api.h>
 #include <atalk/globals.h>
+#include <atalk/logger.h>
 #include <atalk/netatalk_conf.h>
+#include <atalk/paths.h>
+#include <atalk/server_ipc.h>
+
+#ifdef WITH_SPOTLIGHT
 #include <atalk/spotlight.h>
+#endif
+
+#include <atalk/util.h>
+#include <atalk/uuid.h>
 
-#include "switch.h"
 #include "auth.h"
-#include "fork.h"
 #include "dircache.h"
+#include "fork.h"
+#include "switch.h"
 
 #ifndef SOL_TCP
 #define SOL_TCP IPPROTO_TCP
@@ -560,15 +565,15 @@ void afp_over_dsi(AFPObj *obj)
 
             if (debugging) {
                 if (obj->options.logconfig)
-                    setuplog(obj->options.logconfig, obj->options.logfile);
+                    setuplog(obj->options.logconfig, obj->options.logfile, obj->options.log_us_timestamp);
                 else
-                    setuplog("default:note", NULL);
+                    setuplog("default:note", NULL, true);
                 debugging = 0;
             } else {
                 char logstr[50];
                 debugging = 1;
                 sprintf(logstr, "/tmp/afpd.%u.XXXXXX", getpid());
-                setuplog("default:maxdebug", logstr);
+                setuplog("default:maxdebug", logstr, true);
             }
         }
 
@@ -593,14 +598,6 @@ void afp_over_dsi(AFPObj *obj)
             break;
 
         case DSIFUNC_CMD:
-#ifdef AFS
-            if ( writtenfork ) {
-                if ( flushfork( writtenfork ) < 0 ) {
-                    LOG(log_error, logtype_afpd, "main flushfork: %s", strerror(errno) );
-                }
-                writtenfork = NULL;
-            }
-#endif /* AFS */
 
             function = (u_char) dsi->commands[0];
 
