@@ -7,52 +7,47 @@
 #include "config.h"
 #endif /* HAVE_CONFIG_H */
 
+#include <arpa/inet.h>
+#include <ctype.h>
+#include <errno.h>
+#include <grp.h>
+#include <limits.h>
+#include <pwd.h>
 #include <stdio.h>
 #include <stdlib.h>
-#include <errno.h>
-#include <unistd.h>
-#include <sys/types.h>
+#include <string.h>
 #include <sys/param.h>
 #include <sys/stat.h>
-#include <arpa/inet.h>
+#include <sys/types.h>
+#include <time.h>
+#include <unistd.h>
 
 #include <atalk/afp.h>
 #include <atalk/compat.h>
-#include <atalk/util.h>
-#include <limits.h>
-#include <string.h>
-#include <ctype.h>
-#include <time.h>
-#include <pwd.h>
-#include <grp.h>
-
-#ifdef TRU64
-#include <netdb.h>
-#include <arpa/inet.h>
-#include <sia.h>
-#include <siad.h>
-
-extern void afp_get_cmdline( int *ac, char ***av );
-#endif /* TRU64 */
-
+#include <atalk/fce_api.h>
+#include <atalk/globals.h>
 #include <atalk/logger.h>
 #include <atalk/server_ipc.h>
-#include <atalk/uuid.h>
-#include <atalk/globals.h>
-#include <atalk/fce_api.h>
-#include <atalk/spotlight.h>
-#include <atalk/unix.h>
 
-#include "auth.h"
-#include "uam_auth.h"
-#include "switch.h"
-#include "status.h"
-#include "fork.h"
-#include "extattrs.h"
+#ifdef WITH_SPOTLIGHT
+#include <atalk/spotlight.h>
+#endif
+
+#include <atalk/unix.h>
+#include <atalk/util.h>
+#include <atalk/uuid.h>
+
 #ifdef HAVE_ACLS
 #include "acls.h"
 #endif
 
+#include "auth.h"
+#include "extattrs.h"
+#include "fork.h"
+#include "status.h"
+#include "switch.h"
+#include "uam_auth.h"
+
 static int afp_version_index;
 static struct uam_mod uam_modules = {NULL, NULL, &uam_modules, &uam_modules};
 static struct uam_obj uam_login = {"", "", 0, {{NULL, NULL, NULL, NULL }}, &uam_login,
@@ -134,14 +129,14 @@ static int send_reply(const AFPObj *obj, const int err)
     return AFP_OK;
 }
 
-static int afp_errpwdexpired(AFPObj *obj _U_, char *ibuf _U_, size_t ibuflen _U_, 
+static int afp_errpwdexpired(AFPObj *obj _U_, char *ibuf _U_, size_t ibuflen _U_,
                              char *rbuf _U_, size_t *rbuflen)
 {
     *rbuflen = 0;
     return AFPERR_PWDEXPR;
 }
 
-static int afp_null_nolog(AFPObj *obj _U_, char *ibuf _U_, size_t ibuflen _U_, 
+static int afp_null_nolog(AFPObj *obj _U_, char *ibuf _U_, size_t ibuflen _U_,
                           char *rbuf _U_, size_t *rbuflen)
 {
     *rbuflen = 0;
@@ -152,22 +147,7 @@ static int set_auth_switch(const AFPObj *obj, int expired)
 {
     int i;
 
-    if (expired) {
-        /*
-         * BF: expired password handling
-         * to allow the user to change his/her password we have to allow login
-         * but every following call except for FPChangePassword will be thrown
-         * away with an AFPERR_PWDEXPR error. (thanks to Leland Wallace from Apple
-         * for clarifying this)
-         */
-
-        for (i=0; i<=0xff; i++) {
-            uam_afpserver_action(i, UAM_AFPSERVER_PREAUTH, afp_errpwdexpired, NULL);
-        }
-        uam_afpserver_action(AFP_LOGOUT, UAM_AFPSERVER_PREAUTH, afp_logout, NULL);
-        uam_afpserver_action(AFP_CHANGEPW, UAM_AFPSERVER_PREAUTH, afp_changepw, NULL);
-    }
-    else {
+    if (!expired) {
         afp_switch = postauth_switch;
         switch (obj->afp_version) {
 
@@ -187,7 +167,7 @@ static int set_auth_switch(const AFPObj *obj, int expired)
         case 31:
             uam_afpserver_action(AFP_SYNCDIR, UAM_AFPSERVER_POSTAUTH, afp_syncdir, NULL);
             uam_afpserver_action(AFP_SYNCFORK, UAM_AFPSERVER_POSTAUTH, afp_syncfork, NULL);
-#ifdef HAVE_TRACKER
+#ifdef WITH_SPOTLIGHT
             uam_afpserver_action(AFP_SPOTLIGHT_PRIVATE, UAM_AFPSERVER_POSTAUTH, afp_spotlight_rpc, NULL);
 #endif
             uam_afpserver_action(AFP_ENUMERATE_EXT2, UAM_AFPSERVER_POSTAUTH, afp_enumerate_ext2, NULL);
@@ -211,6 +191,20 @@ static int set_auth_switch(const AFPObj *obj, int expired)
             uam_afpserver_action(AFP_ZZZ,  UAM_AFPSERVER_POSTAUTH, afp_zzz, NULL);
             break;
         }
+    } else {
+        /*
+         * BF: expired password handling
+         * to allow the user to change his/her password we have to allow login
+         * but every following call except for FPChangePassword will be thrown
+         * away with an AFPERR_PWDEXPR error. (thanks to Leland Wallace from Apple
+         * for clarifying this)
+         */
+
+        for (i=0; i<=0xff; i++) {
+            uam_afpserver_action(i, UAM_AFPSERVER_PREAUTH, afp_errpwdexpired, NULL);
+        }
+        uam_afpserver_action(AFP_LOGOUT, UAM_AFPSERVER_PREAUTH, afp_logout, NULL);
+        uam_afpserver_action(AFP_CHANGEPW, UAM_AFPSERVER_PREAUTH, afp_changepw, NULL);
     }
 
     return AFP_OK;
@@ -390,7 +384,7 @@ static int create_session_key(AFPObj *obj)
 /* ---------------------- */
 int afp_getsession(
     AFPObj *obj,
-    char   *ibuf, size_t ibuflen, 
+    char   *ibuf, size_t ibuflen,
     char   *rbuf, size_t *rbuflen)
 {
     uint16_t           type;
@@ -835,7 +829,7 @@ int afp_logout(AFPObj *obj, char *ibuf _U_, size_t ibuflen  _U_, char *rbuf  _U_
  */
 int afp_changepw(AFPObj *obj, char *ibuf, size_t ibuflen, char *rbuf, size_t *rbuflen)
 {
-    char username[MACFILELEN + 1], *start = ibuf;
+    char username[MAXUSERLEN], *start = ibuf;
     struct uam_obj *uam;
     struct passwd *pwd;
     size_t len;
@@ -886,9 +880,9 @@ int afp_changepw(AFPObj *obj, char *ibuf, size_t ibuflen, char *rbuf, size_t *rb
         return AFPERR_PARAM;
 
     /* send it off to the uam. we really don't use ibuflen right now. */
-    if (ibuflen < (size_t)(ibuf - start)) 
+    if (ibuflen < (size_t)(ibuf - start))
         return AFPERR_PARAM;
-    
+
     ibuflen -= (ibuf - start);
     ret = uam->u.uam_changepw(obj, username, pwd, ibuf, ibuflen,
                               rbuf, rbuflen);
