@@ -6,13 +6,7 @@
 #ifndef AFPD_AUTH_H
 #define AFPD_AUTH_H 1
 
-#ifdef HAVE_LIMITS_H
 #include <limits.h>
-#endif /* HAVE_LIMITS_H */
-
-#ifdef HAVE_SYS_CDEFS_H
-#include <sys/types.h>
-#endif /* HAVE_SYS_CDEFS_H */
 
 #include <atalk/globals.h>
 
@@ -22,16 +16,12 @@ struct afp_versions {
 };
 
 static const struct afp_versions  afp_versions[] = {
-#ifndef NO_DDP
-    { "AFPVersion 1.1", 11 },
-    { "AFPVersion 2.0", 20 },
-    { "AFPVersion 2.1", 21 },
-#endif /* ! NO_DDP */
     { "AFP2.2", 22 },
     { "AFPX03", 30 },
     { "AFP3.1", 31 },
     { "AFP3.2", 32 },
-    { "AFP3.3", 33 }
+    { "AFP3.3", 33 },
+    { "AFP3.4", 34 }
 };
 
 /* for GetUserInfo */
@@ -44,9 +34,6 @@ extern uid_t    uuid;
 extern gid_t	*groups;
 extern int	ngroups;
 
-extern int set_groups(AFPObj *obj, struct passwd *pwd);
-extern const char *print_groups(int ngroups, gid_t *groups);
-
 /* FP functions */
 int afp_login (AFPObj *obj, char *ibuf, size_t ibuflen, char *rbuf,  size_t *rbuflen);
 int afp_login_ext (AFPObj *obj, char *ibuf, size_t ibuflen, char *rbuf,  size_t *rbuflen);
