@@ -1,4 +1,4 @@
-/* 
+/*
  * Netatalk 2002 (c)
  * Copyright (C) 1990, 1993 Regents of The University of Michigan
  * Copyright (C) 2010 Frank Lahm
@@ -21,51 +21,51 @@
  *
  * Starting with Netatalk 2.2 searching by name criteria utilizes the
  * CNID database in conjunction with an enhanced cnid_dbd. This requires
- * the use of cnidscheme:dbd for the searched volume, the new functionality
- * is not built into cnidscheme:cdb.
+ * the use of cnidscheme:dbd for the searched volume.
  */
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif /* HAVE_CONFIG_H */
 
-#include <sys/types.h>
-#include <sys/stat.h>
-#include <unistd.h>
+#include <ctype.h>
+#include <errno.h>
+#include <netinet/in.h>
 #include <stdio.h>
 #include <stdlib.h>
-#include <errno.h>
-#include <ctype.h>
 #include <string.h>
-#include <time.h>
 #include <string.h>
 #include <sys/file.h>
-#include <netinet/in.h>
+#include <sys/stat.h>
+#include <sys/types.h>
+#include <time.h>
+#include <unistd.h>
 
-#include <atalk/afp.h>
 #include <atalk/adouble.h>
-#include <atalk/logger.h>
-#include <atalk/cnid.h>
-#include <atalk/cnid_dbd_private.h>
-#include <atalk/util.h>
+#include <atalk/afp.h>
 #include <atalk/bstradd.h>
-#include <atalk/unicode.h>
+#include <atalk/cnid_bdb_private.h>
+#include <atalk/cnid.h>
 #include <atalk/globals.h>
+#include <atalk/logger.h>
+#include <atalk/netatalk_conf.h>
+#include <atalk/unicode.h>
+#include <atalk/util.h>
 
 #include "desktop.h"
-#include "directory.h"
 #include "dircache.h"
+#include "directory.h"
 #include "file.h"
-#include "volume.h"
 #include "filedir.h"
 #include "fork.h"
+#include "volume.h"
 
 
 struct finderinfo {
-	u_int32_t f_type;
-	u_int32_t creator;
-	u_int16_t attrs;    /* File attributes (high 8 bits)*/
-	u_int16_t label;    /* Label (low 8 bits )*/
+	uint32_t f_type;
+	uint32_t creator;
+	uint16_t attrs;    /* File attributes (high 8 bits)*/
+	uint16_t label;    /* Label (low 8 bits )*/
 	char reserved[22]; /* Unknown (at least for now...) */
 };
 
@@ -89,16 +89,16 @@ typedef char packed_finder[ADEDLEN_FINDERI];
 
 /* This is our search-criteria structure. */
 struct scrit {
-	u_int32_t rbitmap;          /* Request bitmap - which values should we check ? */
-	u_int16_t fbitmap, dbitmap; /* file & directory bitmap - which values should we return ? */
-	u_int16_t attr;             /* File attributes */
+	uint32_t rbitmap;          /* Request bitmap - which values should we check ? */
+	uint16_t fbitmap, dbitmap; /* file & directory bitmap - which values should we return ? */
+	uint16_t attr;             /* File attributes */
 	time_t cdate;               /* Creation date */
 	time_t mdate;               /* Last modification date */
 	time_t bdate;               /* Last backup date */
-	u_int32_t pdid;             /* Parent DID */
-	u_int16_t offcnt;           /* Offspring count */
+	uint32_t pdid;             /* Parent DID */
+    uint16_t offcnt;           /* Offspring count */
 	struct finderinfo finfo;    /* Finder info */
-	char lname[64];             /* Long name */ 
+	char lname[64];             /* Long name */
 	char utf8name[514];         /* UTF8 or UCS2 name */ /* for convert_charset dest_len parameter +2 */
 };
 
@@ -112,9 +112,9 @@ struct scrit {
  */
 struct dsitem {
     cnid_t ds_did;               /* CNID of this directory           */
-    u_int32_t ds_checked;    /* Have we checked this directory ? */
+    uint32_t ds_checked;    /* Have we checked this directory ? */
 };
- 
+
 
 #define DS_BSIZE 128
 static int save_cidx = -1; /* Saved index of currently scanned directory. */
@@ -124,7 +124,7 @@ static int dsidx = 0;   	     /* First free item index... */
 static struct scrit c1, c2;          /* search criteria */
 
 /* Clears directory stack. */
-static void clearstack(void) 
+static void clearstack(void)
 {
 	save_cidx = -1;
 	while (dsidx > 0) {
@@ -136,13 +136,12 @@ static void clearstack(void)
 static int addstack(char *uname, struct dir *dir, int pidx)
 {
 	struct dsitem *ds;
-	size_t         l, u;
     struct dsitem *tmpds = NULL;
 
 	/* check if we have some space on stack... */
 	if (dsidx >= dssize) {
 		dssize += DS_BSIZE;
-		tmpds = realloc(dstack, dssize * sizeof(struct dsitem));	
+		tmpds = realloc(dstack, dssize * sizeof(struct dsitem));
 		if (tmpds == NULL) {
             clearstack();
             free(dstack);
@@ -174,37 +173,37 @@ static int reducestack(void)
 //			free(dstack[dsidx].path);
 		} else
 			return dsidx - 1;
-	} 
+	}
 	return -1;
-} 
+}
 
-/* Looks up for an opened adouble structure, opens resource fork of selected file. 
+/* Looks up for an opened adouble structure, opens resource fork of selected file.
  * FIXME What about noadouble?
 */
 static struct adouble *adl_lkup(struct vol *vol, struct path *path, struct adouble *adp)
 {
 	static struct adouble ad;
-	
+
 	struct ofork *of;
 	int isdir;
-	
+
 	if (adp)
 	    return adp;
-	    
+
 	isdir  = S_ISDIR(path->st.st_mode);
 
 	if (!isdir && (of = of_findname(vol, path))) {
 		adp = of->of_ad;
 	} else {
-		ad_init(&ad, vol->v_adouble, vol->v_ad_options);
+		ad_init(&ad, vol);
 		adp = &ad;
-	} 
+	}
 
     if ( ad_metadata( path->u_name, ((isdir) ? ADFLAGS_DIR : 0), adp) < 0 ) {
         adp = NULL; /* FIXME without resource fork adl_lkup will be call again */
     }
-    
-	return adp;	
+
+	return adp;
 }
 
 /* -------------------- */
@@ -226,7 +225,7 @@ unpack_finderinfo(struct vol *vol, struct path *path, struct adouble **adp, stru
 {
 	packed_finder  buf;
 	void           *ptr;
-	
+
     *adp = adl_lkup(vol, path, *adp);
 	ptr = get_finderinfo(vol, path->u_name, *adp, &buf,islnk);
 	return unpack_buffer(finfo, ptr);
@@ -237,17 +236,17 @@ unpack_finderinfo(struct vol *vol, struct path *path, struct adouble **adp, stru
 /* Criteria checker. This function returns a 2-bit value. */
 /* bit 0 means if checked file meets given criteria. */
 /* bit 1 means if it is a directory and we should descent into it. */
-/* uname - UNIX name 
+/* uname - UNIX name
  * fname - our fname (translated to UNIX)
  * cidx - index in directory stack
  */
 static int crit_check(struct vol *vol, struct path *path) {
 	int result = 0;
-	u_int16_t attr, flags = CONV_PRECOMPOSE;
+	uint16_t attr, flags = CONV_PRECOMPOSE;
 	struct finderinfo *finfo = NULL, finderinfo;
 	struct adouble *adp = NULL;
 	time_t c_date, b_date;
-	u_int32_t ac_date, ab_date;
+	uint32_t ac_date, ab_date;
 	static char convbuf[514]; /* for convert_charset dest_len parameter +2 */
 	size_t len;
     int islnk;
@@ -261,16 +260,16 @@ static int crit_check(struct vol *vol, struct path *path) {
 		if (!c1.fbitmap)
 			return 0;
 
-		/* compute the Mac name 
+		/* compute the Mac name
 		 * first try without id (it's slow to find it)
-		 * An other option would be to call get_id in utompath but 
+		 * An other option would be to call get_id in utompath but
 		 * we need to pass parent dir
 		*/
-        if (!(path->m_name = utompath(vol, path->u_name, 0 , utf8_encoding()) )) {
+        if (!(path->m_name = utompath(vol, path->u_name, 0 , utf8_encoding(vol->v_obj)) )) {
         	/*retry with the right id */
-       
+
         	cnid_t id;
-        	
+
         	adp = adl_lkup(vol, path, adp);
         	id = get_id(vol, adp, &path->st, path->d_dir->d_did, path->u_name, strlen(path->u_name));
         	if (!id) {
@@ -279,20 +278,20 @@ static int crit_check(struct vol *vol, struct path *path) {
         	}
         	/* save the id for getfilparm */
         	path->id = id;
-        	if (!(path->m_name = utompath(vol, path->u_name, id , utf8_encoding()))) {
+        	if (!(path->m_name = utompath(vol, path->u_name, id , utf8_encoding(vol->v_obj)))) {
         		return 0;
         	}
         }
 	}
-		
-	/* Kind of optimization: 
+
+	/* Kind of optimization:
 	 * -- first check things we've already have - filename
 	 * -- last check things we get from ad_open()
 	 * FIXME strmcp strstr (icase)
 	 */
 
 	/* Check for filename */
-	if ((c1.rbitmap & (1U<<DIRPBIT_LNAME))) { 
+	if ((c1.rbitmap & (1U<<DIRPBIT_LNAME))) {
 		if ( (size_t)(-1) == (len = convert_string(vol->v_maccharset, CH_UCS2, path->m_name, -1, convbuf, 512)) )
 			goto crit_check_ret;
 
@@ -302,9 +301,9 @@ static int crit_check(struct vol *vol, struct path *path) {
 		} else
 			if (strcasecmp_w((ucs2_t*) convbuf, (ucs2_t*) c1.lname) != 0)
 				goto crit_check_ret;
-	} 
-	
-	if ((c1.rbitmap & (1U<<FILPBIT_PDINFO))) { 
+	}
+
+	if ((c1.rbitmap & (1U<<FILPBIT_PDINFO))) {
 		if ( (size_t)(-1) == (len = convert_charset( CH_UTF8_MAC, CH_UCS2, CH_UTF8, path->m_name, strlen(path->m_name), convbuf, 512, &flags))) {
 			goto crit_check_ret;
 		}
@@ -315,7 +314,7 @@ static int crit_check(struct vol *vol, struct path *path) {
 		} else
 			if (strcasecmp_w((ucs2_t *)convbuf, (ucs2_t*)c1.utf8name) != 0)
 				goto crit_check_ret;
-	} 
+	}
 
 
 	/* FIXME */
@@ -331,7 +330,7 @@ static int crit_check(struct vol *vol, struct path *path) {
 		if (path->st.st_mtime < c1.mdate || path->st.st_mtime > c2.mdate)
 			goto crit_check_ret;
 	}
-	
+
 	/* Check for creation date... */
 	if ((c1.rbitmap & (1U<<DIRPBIT_CDATE))) {
 		c_date = path->st.st_mtime;
@@ -353,16 +352,16 @@ static int crit_check(struct vol *vol, struct path *path) {
 		if (b_date < c1.bdate || b_date > c2.bdate)
 			goto crit_check_ret;
 	}
-				
+
 	/* Check attributes */
 	if ((c1.rbitmap & (1U<<DIRPBIT_ATTR)) && c2.attr != 0) {
 		if ((adp = adl_lkup(vol, path, adp))) {
 			ad_getattr(adp, &attr);
 			if ((attr & c2.attr) != c1.attr)
 				goto crit_check_ret;
-		} else 
+		} else
 			goto crit_check_ret;
-	}		
+	}
 
         /* Check file type ID */
 	if ((c1.rbitmap & (1U<<DIRPBIT_FINFO)) && c2.finfo.f_type != 0) {
@@ -370,7 +369,7 @@ static int crit_check(struct vol *vol, struct path *path) {
 		if (finfo->f_type != c1.finfo.f_type)
 			goto crit_check_ret;
 	}
-	
+
 	/* Check creator ID */
 	if ((c1.rbitmap & (1U<<DIRPBIT_FINFO)) && c2.finfo.creator != 0) {
 		if (!finfo) {
@@ -379,7 +378,7 @@ static int crit_check(struct vol *vol, struct path *path) {
 		if (finfo->creator != c1.finfo.creator)
 			goto crit_check_ret;
 	}
-		
+
 	/* Check finder info attributes */
 	if ((c1.rbitmap & (1U<<DIRPBIT_FINFO)) && c2.finfo.attrs != 0) {
 		if (!finfo) {
@@ -389,7 +388,7 @@ static int crit_check(struct vol *vol, struct path *path) {
 		if ((finfo->attrs & c2.finfo.attrs) != c1.finfo.attrs)
 			goto crit_check_ret;
 	}
-	
+
 	/* Check label */
 	if ((c1.rbitmap & (1U<<DIRPBIT_FINFO)) && c2.finfo.label != 0) {
 		if (!finfo) {
@@ -397,30 +396,30 @@ static int crit_check(struct vol *vol, struct path *path) {
 		}
 		if ((finfo->label & c2.finfo.label) != c1.finfo.label)
 			goto crit_check_ret;
-	}	
+	}
 	/* FIXME: Attributes check ! */
-	
+
 	/* All criteria are met. */
 	result |= 1;
 crit_check_ret:
 	if (adp != NULL)
-		ad_close_metadata(adp);
+		ad_close(adp, ADFLAGS_HF);
 	return result;
-}  
+}
 
 /* ------------------------------ */
-static int rslt_add ( struct vol *vol, struct path *path, char **buf, int ext)
+static int rslt_add (const AFPObj *obj, struct vol *vol, struct path *path, char **buf, int ext)
 {
 
 	char 		*p = *buf;
 	int 		ret;
 	size_t		tbuf =0;
-	u_int16_t	resultsize;
-	int 		isdir = S_ISDIR(path->st.st_mode); 
+	uint16_t	resultsize;
+	int 		isdir = S_ISDIR(path->st.st_mode);
 
 	/* Skip resultsize */
 	if (ext) {
-		p += sizeof(resultsize); 
+		p += sizeof(resultsize);
 	}
 	else {
 		p++;
@@ -430,13 +429,13 @@ static int rslt_add ( struct vol *vol, struct path *path, char **buf, int ext)
 	if (ext) {
 		*p++ = 0;                  /* Pad */
 	}
-	
+
 	if ( isdir ) {
-        ret = getdirparams(vol, c1.dbitmap, path, path->d_dir, p , &tbuf ); 
+        ret = getdirparams(obj, vol, c1.dbitmap, path, path->d_dir, p , &tbuf );
 	}
 	else {
 	    /* FIXME slow if we need the file ID, we already know it, done ? */
-		ret = getfilparams ( vol, c1.fbitmap, path, path->d_dir, p, &tbuf);
+		ret = getfilparams (obj, vol, c1.fbitmap, path, path->d_dir, p, &tbuf, 0);
 	}
 
 	if ( ret != AFP_OK )
@@ -459,8 +458,8 @@ static int rslt_add ( struct vol *vol, struct path *path, char **buf, int ext)
 	}
 
 	return 1;
-} 
-	
+}
+
 #define VETO_STR \
         "./../.AppleDouble/.AppleDB/Network Trash Folder/TheVolumeSettingsFolder/TheFindByContentFolder/.AppleDesktop/.Parent/"
 
@@ -479,8 +478,9 @@ static int rslt_add ( struct vol *vol, struct path *path, char **buf, int ext)
  * @param ext       (r)  extended search flag
  */
 #define NUM_ROUNDS 200
-static int catsearch(struct vol *vol,
-                     struct dir *dir,  
+static int catsearch(const AFPObj *obj,
+                     struct vol *vol,
+                     struct dir *dir,
                      int rmatches,
                      uint32_t *pos,
                      char *rbuf,
@@ -488,7 +488,7 @@ static int catsearch(struct vol *vol,
                      int *rsize,
                      int ext)
 {
-    static u_int32_t cur_pos;    /* Saved position index (ID) - used to remember "position" across FPCatSearch calls */
+    static uint32_t cur_pos;    /* Saved position index (ID) - used to remember "position" across FPCatSearch calls */
     static DIR *dirpos; 		 /* UNIX structure describing currently opened directory. */
     struct dir *currentdir;      /* struct dir of current directory */
 	int cidx, r;
@@ -518,8 +518,8 @@ static int catsearch(struct vol *vol,
 		if (dirpos != NULL) {
 			closedir(dirpos);
 			dirpos = NULL;
-		} 
-		
+		}
+
 		if (addstack(vpath, dir, -1) == -1) {
 			result = AFPERR_MISC;
 			goto catsearch_end;
@@ -532,7 +532,7 @@ static int catsearch(struct vol *vol,
         result = AFPERR_MISC;
         goto catsearch_end;
     }
-	
+
 	/* So we are beginning... */
     start_time = time(NULL);
 
@@ -549,7 +549,7 @@ static int catsearch(struct vol *vol,
 			dirpos = opendir(".");
 
 		if (dirpos == NULL) {
-			const char *temp = bdata(currentdir->d_fullpath);
+			const char *temp = cfrombstr(currentdir->d_fullpath);
 			dirpos = opendir(temp);
 		}
 
@@ -571,7 +571,7 @@ static int catsearch(struct vol *vol,
 			goto catsearch_end;
 		}
 
-		
+
 		while ((entry = readdir(dirpos)) != NULL) {
 			(*pos)++;
 
@@ -596,12 +596,12 @@ static int catsearch(struct vol *vol,
 				default:
 					result = AFPERR_MISC;
 					goto catsearch_end;
-				} 
+				}
 			}
             switch (S_IFMT & path.st.st_mode) {
             case S_IFDIR:
-				/* here we can short cut 
-				   ie if in the same loop the parent dir wasn't in the cache
+				/* here we can short cut
+				   i.e. if in the same loop the parent dir wasn't in the cache
 				   ALL dirsearch_byname will fail.
 				*/
                 unlen = strlen(path.u_name);
@@ -620,11 +620,11 @@ static int catsearch(struct vol *vol,
 					}
                 }
                 path.m_name = cfrombstr(path.d_dir->d_m_name);
-                	
+
 				if (addstack(path.u_name, path.d_dir, cidx) == -1) {
 					result = AFPERR_MISC;
 					goto catsearch_end;
-				} 
+				}
                 break;
             case S_IFREG:
             	path.d_dir = currentdir;
@@ -637,15 +637,15 @@ static int catsearch(struct vol *vol,
 
 			/* bit 0 means that criteria has been met */
 			if ((ccr & 1)) {
-				r = rslt_add ( vol, &path, &rrbuf, ext);
-				
+				r = rslt_add (obj, vol, &path, &rrbuf, ext);
+
 				if (r == 0) {
 					result = AFPERR_MISC;
 					goto catsearch_end;
-				} 
+				}
 				*nrecs += r;
 				/* Number of matches limit */
-				if (--rmatches == 0) 
+				if (--rmatches == 0)
 					goto catsearch_pause;
 				/* Block size limit */
 				if (rrbuf - rbuf >= 448)
@@ -670,14 +670,14 @@ static int catsearch(struct vol *vol,
 	goto catsearch_end;
 
 catsearch_pause:
-	cur_pos = *pos; 
+	cur_pos = *pos;
 	save_cidx = cidx;
 
 catsearch_end: /* Exiting catsearch: error condition */
 	*rsize = rrbuf - rbuf;
     if (cwd != -1) {
         if ((fchdir(cwd)) != 0) {
-            LOG(log_debug, logtype_afpd, "error chdiring back: %s", strerror(errno));        
+            LOG(log_debug, logtype_afpd, "error chdiring back: %s", strerror(errno));
         }
         close(cwd);
     }
@@ -699,8 +699,9 @@ catsearch_end: /* Exiting catsearch: error condition */
  * @param rsize     (w)  length of data written to output buffer
  * @param ext       (r)  extended search flag
  */
-static int catsearch_db(struct vol *vol,
-                        struct dir *dir,  
+static int catsearch_db(const AFPObj *obj,
+                        struct vol *vol,
+                        struct dir *dir,
                         const char *uname,
                         int rmatches,
                         uint32_t *pos,
@@ -721,7 +722,7 @@ static int catsearch_db(struct vol *vol,
 
     LOG(log_debug, logtype_afpd, "catsearch_db(req pos: %u): {pos: %u, name: %s}",
         *pos, cur_pos, uname);
-        
+
 	if (*pos != 0 && *pos != cur_pos) {
 		result = AFPERR_CATCHNG;
 		goto catsearch_end;
@@ -743,16 +744,19 @@ static int catsearch_db(struct vol *vol,
 
         LOG(log_debug, logtype_afpd, "catsearch_db: %s", buffer);
 
-        if ((num_matches = cnid_find(vol->v_cdb,
-                                     buffer,
-                                     strlen(uname),
-                                     resbuf,
-                                     sizeof(resbuf))) == -1) {
+        AFP_CNID_START("cnid_find");
+        num_matches = cnid_find(vol->v_cdb,
+                                buffer,
+                                strlen(uname),
+                                resbuf,
+                                sizeof(resbuf));
+        AFP_CNID_DONE();
+        if (num_matches == -1) {
             result = AFPERR_MISC;
             goto catsearch_end;
         }
     }
-	
+
 	while (cur_pos < num_matches) {
         char *name;
         cnid_t cnid, did;
@@ -763,8 +767,12 @@ static int catsearch_db(struct vol *vol,
         memcpy(&cnid, resbuf + cur_pos * sizeof(cnid_t), sizeof(cnid_t));
         did = cnid;
 
-        if ((name = cnid_resolve(vol->v_cdb, &did, resolvebuf, 12 + MAXPATHLEN + 1)) == NULL)
+        AFP_CNID_START("cnid_resolve");
+        name = cnid_resolve(vol->v_cdb, &did, resolvebuf, 12 + MAXPATHLEN + 1);
+        AFP_CNID_DONE();
+        if (name == NULL)
             goto next;
+
         LOG(log_debug, logtype_afpd, "catsearch_db: {pos: %u, name:%s, cnid: %u}",
             cur_pos, name, ntohl(cnid));
         if ((dir = dirlookup(vol, did)) == NULL)
@@ -774,7 +782,7 @@ static int catsearch_db(struct vol *vol,
 
         memset(&path, 0, sizeof(path));
         path.u_name = name;
-        path.m_name = utompath(vol, name, cnid, utf8_encoding());
+        path.m_name = utompath(vol, name, cnid, utf8_encoding(vol->v_obj));
 
         if (of_stat(vol, &path) != 0) {
             switch (errno) {
@@ -782,11 +790,11 @@ static int catsearch_db(struct vol *vol,
             case ELOOP:
                 goto next;
             case ENOENT:
-                
+
             default:
                 result = AFPERR_MISC;
                 goto catsearch_end;
-            } 
+            }
         }
         /* For files path.d_dir is the parent dir, for dirs its the dir itself */
         if (S_ISDIR(path.st.st_mode))
@@ -794,7 +802,7 @@ static int catsearch_db(struct vol *vol,
                 goto next;
         path.d_dir = dir;
 
-        LOG(log_maxdebug, logtype_afpd,"catsearch_db: dir: %s, cwd: %s, name: %s", 
+        LOG(log_maxdebug, logtype_afpd,"catsearch_db: dir: %s, cwd: %s, name: %s",
             cfrombstr(dir->d_fullpath), getcwdpath(), path.u_name);
 
         /* At last we can check the search criteria */
@@ -803,14 +811,14 @@ static int catsearch_db(struct vol *vol,
             LOG(log_debug, logtype_afpd,"catsearch_db: match: %s/%s",
                 getcwdpath(), path.u_name);
             /* bit 1 means that criteria has been met */
-            r = rslt_add(vol, &path, &rrbuf, ext);
+            r = rslt_add(obj, vol, &path, &rrbuf, ext);
             if (r == 0) {
                 result = AFPERR_MISC;
                 goto catsearch_end;
-            } 
+            }
             *nrecs += r;
             /* Number of matches limit */
-            if (--rmatches == 0) 
+            if (--rmatches == 0)
                 goto catsearch_pause;
             /* Block size limit */
             if (rrbuf - rbuf >= 448)
@@ -839,19 +847,19 @@ static int catsearch_afp(AFPObj *obj _U_, char *ibuf, size_t ibuflen,
                   char *rbuf, size_t *rbuflen, int ext)
 {
     struct vol *vol;
-    u_int16_t   vid;
-    u_int16_t   spec_len;
-    u_int32_t   rmatches, reserved;
-    u_int32_t	catpos[4];
-    u_int32_t   pdid = 0;
+    uint16_t   vid;
+    uint16_t   spec_len;
+    uint32_t   rmatches, reserved;
+    uint32_t	catpos[4];
+    uint32_t   pdid = 0;
     int ret, rsize;
-    u_int32_t nrecs = 0;
+    uint32_t nrecs = 0;
     unsigned char *spec1, *spec2, *bspec1, *bspec2;
     size_t	len;
-    u_int16_t	namelen;
-    u_int16_t	flags;
+    uint16_t	namelen;
+    uint16_t	flags;
     char  	    tmppath[256];
-    char        *uname = NULL;
+    const char  *uname = NULL;
 
     *rbuflen = 0;
 
@@ -870,10 +878,10 @@ static int catsearch_afp(AFPObj *obj _U_, char *ibuf, size_t ibuflen,
     if ((vol = getvolbyvid(vid)) == NULL) {
         return AFPERR_PARAM;
     }
-    
+
     memcpy(&rmatches, ibuf, sizeof(rmatches));
     rmatches = ntohl(rmatches);
-    ibuf += sizeof(rmatches); 
+    ibuf += sizeof(rmatches);
 
     /* FIXME: (rl) should we check if reserved == 0 ? */
     ibuf += sizeof(reserved);
@@ -912,8 +920,8 @@ static int catsearch_afp(AFPObj *obj _U_, char *ibuf, size_t ibuflen,
     spec1 = (unsigned char*)ibuf;
     spec2 = (unsigned char*)ibuf + spec_len + 2;
 
-    spec1 += 2; 
-    spec2 += 2; 
+    spec1 += 2;
+    spec2 += 2;
 
     bspec1 = spec1;
     bspec2 = spec2;
@@ -953,7 +961,7 @@ static int catsearch_afp(AFPObj *obj _U_, char *ibuf, size_t ibuflen,
 	    c2.mdate = AD_DATE_TO_UNIX(c2.mdate);
 	    spec2 += sizeof(c1.mdate);
     }
-    
+
     /* Backup date */
     if (c1.rbitmap & (1U << FILPBIT_BDATE)) {
 	    memcpy(&c1.bdate, spec1, sizeof(c1.bdate));
@@ -967,9 +975,9 @@ static int catsearch_afp(AFPObj *obj _U_, char *ibuf, size_t ibuflen,
     /* Finder info */
     if (c1.rbitmap & (1U << FILPBIT_FINFO)) {
     	packed_finder buf;
-    	
+
 	    memcpy(buf, spec1, sizeof(buf));
-	    unpack_buffer(&c1.finfo, buf);    	
+	    unpack_buffer(&c1.finfo, buf);
 	    spec1 += sizeof(buf);
 
 	    memcpy(buf, spec2, sizeof(buf));
@@ -997,14 +1005,14 @@ static int catsearch_afp(AFPObj *obj _U_, char *ibuf, size_t ibuflen,
 
     /* Long name */
     if (c1.rbitmap & (1U << FILPBIT_LNAME)) {
-        /* Get the long filename */	
+        /* Get the long filename */
 		memcpy(tmppath, bspec1 + spec1[1] + 1, (bspec1 + spec1[1])[0]);
 		tmppath[(bspec1 + spec1[1])[0]]= 0;
 		len = convert_string ( vol->v_maccharset, CH_UCS2, tmppath, -1, c1.lname, sizeof(c1.lname));
         if (len == (size_t)(-1))
             return AFPERR_PARAM;
 
-#if 0	
+#if 0
 		/* FIXME: do we need it ? It's always null ! */
 		memcpy(c2.lname, bspec2 + spec2[1] + 1, (bspec2 + spec2[1])[0]);
 		c2.lname[(bspec2 + spec2[1])[0]]= 0;
@@ -1027,7 +1035,7 @@ static int catsearch_afp(AFPObj *obj _U_, char *ibuf, size_t ibuflen,
 
 		memcpy (c1.utf8name, spec1+2, namelen);
 		c1.utf8name[namelen] = 0;
-        if ((uname = mtoupath(vol, c1.utf8name, 0, utf8_encoding())) == NULL)
+        if ((uname = mtoupath(vol, c1.utf8name, 0, utf8_encoding(obj))) == NULL)
             return AFPERR_PARAM;
 
  		/* convert charset */
@@ -1036,7 +1044,7 @@ static int catsearch_afp(AFPObj *obj _U_, char *ibuf, size_t ibuflen,
         if (len == (size_t)(-1))
             return AFPERR_PARAM;
     }
-    
+
     /* Call search */
     *rbuflen = 24;
     if ((c1.rbitmap & (1U << FILPBIT_PDINFO))
@@ -1044,10 +1052,10 @@ static int catsearch_afp(AFPObj *obj _U_, char *ibuf, size_t ibuflen,
         && (strcmp(vol->v_cnidscheme, "dbd") == 0)
         && (vol->v_flags & AFPVOL_SEARCHDB))
         /* we've got a name and it's a dbd volume, so search CNID database */
-        ret = catsearch_db(vol, vol->v_root, uname, rmatches, &catpos[0], rbuf+24, &nrecs, &rsize, ext);
+        ret = catsearch_db(obj, vol, vol->v_root, uname, rmatches, &catpos[0], rbuf+24, &nrecs, &rsize, ext);
     else
         /* perform a slow filesystem tree search */
-        ret = catsearch(vol, vol->v_root, rmatches, &catpos[0], rbuf+24, &nrecs, &rsize, ext);
+        ret = catsearch(obj, vol, vol->v_root, rmatches, &catpos[0], rbuf+24, &nrecs, &rsize, ext);
 
     memcpy(rbuf, catpos, sizeof(catpos));
     rbuf += sizeof(catpos);
@@ -1083,5 +1091,5 @@ int afp_catsearch_ext (AFPObj *obj, char *ibuf, size_t ibuflen,
 }
 
 /* FIXME: we need a clean separation between afp stubs and 'real' implementation */
-/* (so, all buffer packing/unpacking should be done in stub, everything else 
+/* (so, all buffer packing/unpacking should be done in stub, everything else
    should be done in other functions) */
