@@ -21,18 +21,18 @@
  *	netatalk@itd.umich.edu
  */
 
-#ifndef AFPD_FILE_H 
+#ifndef AFPD_FILE_H
 #define AFPD_FILE_H 1
 
-/*#include <sys/stat.h>*/ /* including it here causes some confusion */
+#include <arpa/inet.h>
 #include <sys/param.h>
-#include <sys/types.h>
-#include <netatalk/endian.h>
+/*#include <sys/stat.h>*/ /* including it here causes some confusion */
+
 #include <atalk/adouble.h>
 #include <atalk/globals.h>
 
-#include "volume.h"
 #include "directory.h"
+#include "volume.h"
 
 extern const u_char	ufinderi[];
 
@@ -52,12 +52,6 @@ extern const u_char	ufinderi[];
 #define FILPBIT_EXTRFLEN 14
 #define FILPBIT_UNIXPR   15
 
-struct extmap {
-    char		*em_ext;
-    char		em_creator[ 4 ];
-    char		em_type[ 4 ];
-};
-
 #define kTextEncodingUTF8 0x08000103
 
 typedef enum {
@@ -104,20 +98,19 @@ typedef enum {
   kTextEncodingMacKeyboardGlyphs = 41,
 } kTextEncoding_t;
 
-extern char *set_name   (const struct vol *, char *, cnid_t, char *, cnid_t, u_int32_t );
+extern char *set_name   (const struct vol *, char *, cnid_t, char *, cnid_t, uint32_t );
 
 extern struct extmap	*getextmap (const char *);
 extern struct extmap	*getdefextmap (void);
 
-extern int getfilparams (struct vol *, u_int16_t, struct path *,
-                                 struct dir *, char *buf, size_t *);
-
-extern int setfilparams (struct vol *, struct path *, u_int16_t, char *);
-extern int renamefile   (const struct vol *, int, char *, char *, char *, struct adouble *);
-extern int copyfile     (const struct vol *, const struct vol *, int, char *, char *, char *, struct adouble *);
+extern int getfilparams (const AFPObj *obj, struct vol *, uint16_t, struct path *,
+                         struct dir *, char *buf, size_t *, int);
+extern int setfilparams (const AFPObj *obj, struct vol *, struct path *, uint16_t, char *);
+extern int renamefile   (struct vol *, struct dir *, int, char *, char *, char *, struct adouble *);
+extern int copyfile     (struct vol *, struct vol *, struct dir *, int, char *, char *, char *, struct adouble *);
 extern int deletefile   (const struct vol *, int, char *, int);
 
-extern int getmetadata  (struct vol *vol, u_int16_t bitmap, struct path *path, 
+extern int getmetadata  (const AFPObj *obj, struct vol *vol, uint16_t bitmap, struct path *path,
                          struct dir *dir, char *buf, size_t *buflen, struct adouble *adp);
 
 extern void *get_finderinfo (const struct vol *, const char *, struct adouble *, void *, int);
