@@ -1,9 +1,10 @@
 #ifndef AFPD_FILEDIR_H
 #define AFPD_FILEDIR_H 1
 
-#include <sys/types.h>
 #include <sys/stat.h>
+
 #include <atalk/globals.h>
+
 #include "volume.h"
 
 extern struct afp_options default_options;
@@ -15,6 +16,8 @@ extern char		*absupath (const struct vol *, struct dir *,
 extern int		veto_file (const char *veto_str, const char *path);
 extern int 		check_name (const struct vol *vol, char *name);
 
+extern int delete_vetoed_files(struct vol *vol, const char *upath, bool in_vetodir);
+
 /* FP functions */
 int afp_moveandrename (AFPObj *obj, char *ibuf, size_t ibuflen, char *rbuf,  size_t *rbuflen);
 int afp_rename (AFPObj *obj, char *ibuf, size_t ibuflen, char *rbuf,  size_t *rbuflen);
