@@ -1,4 +1,5 @@
 /*
+ *
  * Copyright (c) 1990,1993 Regents of The University of Michigan.
  * All Rights Reserved.  See COPYRIGHT.
  */
@@ -18,9 +19,7 @@
 #include <sys/time.h>
 #include <sys/param.h>
 #include <unistd.h>
-#ifdef HAVE_FCNTL_H
 #include <fcntl.h>
-#endif /* HAVE_FCNTL_H */
 
 #if defined(HAVE_SYS_MNTTAB_H) || defined(__svr4__)
 #include <sys/mntent.h>
@@ -29,6 +28,7 @@
 #include <atalk/logger.h>
 #include <atalk/afp.h>
 #include <atalk/compat.h>
+#include <atalk/unix.h>
 #include <atalk/util.h>
 
 #include "auth.h"
@@ -40,8 +40,8 @@
 
 
 static int
-getfreespace(struct vol *vol, VolSpace *bfree, VolSpace *btotal,
-    id_t id, int idtype)
+getfreespace(const AFPObj *obj, struct vol *vol, VolSpace *bfree, VolSpace *btotal,
+	     id_t id, int idtype)
 {
 	uid_t prevuid;
 	const char *msg;
@@ -56,18 +56,7 @@ getfreespace(struct vol *vol, VolSpace *bfree, VolSpace *btotal,
 		return -1;
 	}
 
-	prevuid = geteuid();
-	if (prevuid == -1) {
-		LOG(log_info, logtype_afpd, "geteuid(): %s",
-		    strerror(errno));
-		return -1;
-	}
-
-	if ( seteuid( getuid() ) != 0 )  {
-		LOG(log_info, logtype_afpd, "seteuid(): %s",
-		    strerror(errno));
-		return -1;
-	}
+    become_root();
 
 	/*
 	 * In a tidier world we might keep the quotahandle open for longer...
@@ -79,7 +68,7 @@ getfreespace(struct vol *vol, VolSpace *bfree, VolSpace *btotal,
 			seteuid( prevuid );
 			return 0;
 		}
-	   
+
 		LOG(log_info, logtype_afpd, "quota_open(%s): %s", vol->v_path,
 		    strerror(errno));
 		seteuid( prevuid );
@@ -106,7 +95,7 @@ getfreespace(struct vol *vol, VolSpace *bfree, VolSpace *btotal,
 
 	quota_close(qh);
 
-        seteuid( prevuid );
+    unbecome_root();
 
 	if (qv.qv_usage >= qv.qv_hardlimit ||
 	    (qv.qv_usage >= qv.qv_softlimit && now > qv.qv_expiretime)) {
@@ -121,22 +110,22 @@ getfreespace(struct vol *vol, VolSpace *bfree, VolSpace *btotal,
 	return 1;
 }
 
-int uquota_getvolspace( struct vol *vol, VolSpace *bfree, VolSpace *btotal, const u_int32_t bsize)
+int uquota_getvolspace(const AFPObj *obj, struct vol *vol, VolSpace *bfree, VolSpace *btotal, const u_int32_t bsize)
 {
 	int uret, gret;
 	VolSpace ubfree, ubtotal;
 	VolSpace gbfree, gbtotal;
 
-	uret = getfreespace(vol, &ubfree, &ubtotal,
-	    uuid, QUOTA_IDTYPE_USER);
+	uret = getfreespace(obj, vol, &ubfree, &ubtotal,
+	    obj->uid, QUOTA_IDTYPE_USER);
 	if (uret == 1) {
 		LOG(log_info, logtype_afpd, "quota_get(%s, user): %d %d",
 		    vol->v_path, (int)ubfree, (int)ubtotal);
 	}
 
-	if (ngroups >= 1) {
-		gret = getfreespace(vol, &gbfree, &gbtotal,
-		    groups[0], QUOTA_IDTYPE_GROUP);
+	if (obj->ngroups >= 1) {
+		gret = getfreespace(obj, vol, &gbfree, &gbtotal,
+		    obj->groups[0], QUOTA_IDTYPE_GROUP);
 		if (gret == 1) {
 			LOG(log_info, logtype_afpd, "quota_get(%s, group): %d %d",
 			    vol->v_path, (int)gbfree, (int)gbtotal);
@@ -171,10 +160,6 @@ int uquota_getvolspace( struct vol *vol, VolSpace *bfree, VolSpace *btotal, cons
 
 #else /* HAVE_LIBQUOTA */
 
-/*
-#define DEBUG_QUOTA 0
-*/
-
 #define WANT_USER_QUOTA 0
 #define WANT_GROUP_QUOTA 1
 
@@ -187,7 +172,7 @@ int quotactl(int cmd, const char *special, int id, caddr_t addr)
 
 static int overquota( struct dqblk *);
 
-#ifdef linux
+#ifdef __linux__
 
 #ifdef HAVE_LINUX_XQM_H
 #include <linux/xqm.h>
@@ -243,7 +228,7 @@ static void linuxquota_get_api( void )
             kernel_iface = IFACE_VFSV0;
         }
         else if (errno != ENOSYS && errno != ENOTSUP) {
-            /* RedHat 7.1 (2.4.2-2) newquota check 
+            /* RedHat 7.1 (2.4.2-2) newquota check
              * Q_V2_GETSTATS in it's old place, Q_GETQUOTA in the new place
              * (they haven't moved Q_GETSTATS to its new value) */
             int err_stat = 0;
@@ -307,7 +292,7 @@ static int get_linux_quota(int what, char *path, uid_t euser_id, struct dqblk *d
 		r=get_linux_xfs_quota(what, path, euser_id, dp);
 	else
     		r=get_linux_fs_quota(what, path, euser_id, dp);
-    
+
 	return r;
 }
 
@@ -326,13 +311,13 @@ static int get_linux_xfs_quota(int what, char *path, uid_t euser_id, struct dqbl
 	if ((ret = quotactl(QCMD(Q_XGETQUOTA,(what ? GRPQUOTA : USRQUOTA)), path, euser_id, (caddr_t)&D)))
                return ret;
 
-	dqb->bsize = (u_int64_t)512;
-        dqb->dqb_bsoftlimit  = (u_int64_t)D.d_blk_softlimit;
-        dqb->dqb_bhardlimit  = (u_int64_t)D.d_blk_hardlimit;
-        dqb->dqb_ihardlimit  = (u_int64_t)D.d_ino_hardlimit;
-        dqb->dqb_isoftlimit  = (u_int64_t)D.d_ino_softlimit;
-        dqb->dqb_curinodes   = (u_int64_t)D.d_icount;
-        dqb->dqb_curblocks   = (u_int64_t)D.d_bcount; 
+	dqb->bsize = (uint64_t)512;
+        dqb->dqb_bsoftlimit  = (uint64_t)D.d_blk_softlimit;
+        dqb->dqb_bhardlimit  = (uint64_t)D.d_blk_hardlimit;
+        dqb->dqb_ihardlimit  = (uint64_t)D.d_ino_hardlimit;
+        dqb->dqb_isoftlimit  = (uint64_t)D.d_ino_softlimit;
+        dqb->dqb_curinodes   = (uint64_t)D.d_icount;
+        dqb->dqb_curblocks   = (uint64_t)D.d_bcount;
 #endif
        return ret;
 }
@@ -508,7 +493,7 @@ special(char *file, int *nfs)
 
     if (!found)
 	return (NULL);
-#ifdef linux
+#ifdef __linux__
     if (strcmp(mnt->mnt_type, "xfs") == 0)
 	is_xfs = 1;
 #endif
@@ -520,7 +505,7 @@ special(char *file, int *nfs)
 #endif /* __svr4__ */
 
 
-static int getfsquota(struct vol *vol, const int uid, struct dqblk *dq)
+static int getfsquota(const AFPObj *obj, struct vol *vol, const int uid, struct dqblk *dq)
 
 {
 	struct dqblk dqg;
@@ -529,6 +514,9 @@ static int getfsquota(struct vol *vol, const int uid, struct dqblk *dq)
     struct quotctl      qc;
 #endif
 
+#ifdef __linux__
+    memset(dq, 0, sizeof(struct dqblk));
+#endif
     memset(&dqg, 0, sizeof(dqg));
 	
 #ifdef __svr4__
@@ -540,7 +528,6 @@ static int getfsquota(struct vol *vol, const int uid, struct dqblk *dq)
     }
 
 #else /* __svr4__ */
-
 #ifndef USRQUOTA
 #define USRQUOTA   0
 #endif
@@ -553,68 +540,72 @@ static int getfsquota(struct vol *vol, const int uid, struct dqblk *dq)
     * to one group. */
 
 #ifdef BSD4_4
-    if ( seteuid( getuid() ) == 0 ) {
-        if ( quotactl( vol->v_path, QCMD(Q_GETQUOTA,USRQUOTA),
-                       uid, (char *)dq ) != 0 ) {
-            /* try group quotas */
-            if (ngroups >= 1) {
-                if ( quotactl(vol->v_path, QCMD(Q_GETQUOTA, GRPQUOTA),
-                              groups[0], (char *) &dqg) != 0 ) {
-                    seteuid( uid );
-                    return( AFPERR_PARAM );
-                }
+    become_root();
+    if ( quotactl( vol->v_path, QCMD(Q_GETQUOTA,USRQUOTA),
+                    uid, (char *)dq ) != 0 ) {
+        /* try group quotas */
+        if (obj->ngroups >= 1) {
+            if ( quotactl(vol->v_path, QCMD(Q_GETQUOTA, GRPQUOTA),
+                            obj->groups[0], (char *) &dqg) != 0 ) {
+                unbecome_root();
+                return( AFPERR_PARAM );
             }
         }
-        seteuid( uid );
     }
+    unbecome_root();
 
 #else /* BSD4_4 */
     if (get_linux_quota (WANT_USER_QUOTA, vol->v_gvs, uid, dq) !=0) {
-        return( AFPERR_PARAM );
+        LOG(log_debug, logtype_afpd, "user quota did not work!" );
     }
 
     if (get_linux_quota(WANT_GROUP_QUOTA, vol->v_gvs, getegid(),  &dqg) != 0) {
-#ifdef DEBUG_QUOTA
         LOG(log_debug, logtype_afpd, "group quota did not work!" );
-#endif /* DEBUG_QUOTA */
 
 	return AFP_OK; /* no need to check user vs group quota */
     }
 #endif  /* BSD4_4 */
 
-
     /* return either the group quota entry or user quota entry,
        whichever has the least amount of space remaining
     */
 
     /* if user space remaining > group space remaining */
-    if( 
+    if(
         /* if overquota, free space is 0 otherwise hard-current */
-        ( overquota( dq ) ? 0 : ( dq->dqb_bhardlimit ? dq->dqb_bhardlimit - 
-                                  dq->dqb_curblocks : ~((u_int64_t) 0) ) )
+        ( overquota( dq ) ? 0 : ( dq->dqb_bhardlimit ? dq->dqb_bhardlimit -
+                                  dq->dqb_curblocks : ~((uint64_t) 0) ) )
 
       >
-        
-        ( overquota( &dqg ) ? 0 : ( dqg.dqb_bhardlimit ? dqg.dqb_bhardlimit - 
-                                    dqg.dqb_curblocks : ~((u_int64_t) 0) ) )
+
+        ( overquota( &dqg ) ? 0 : ( dqg.dqb_bhardlimit ? dqg.dqb_bhardlimit -
+                                    dqg.dqb_curblocks : ~((uint64_t) 0) ) )
 
       ) /* if */
     {
         /* use group quota limits rather than user limits */
-        dq->dqb_curblocks = dqg.dqb_curblocks;
         dq->dqb_bhardlimit = dqg.dqb_bhardlimit;
         dq->dqb_bsoftlimit = dqg.dqb_bsoftlimit;
+        dq->dqb_curblocks = dqg.dqb_curblocks;
+#ifdef __linux__
+        dq->dqb_ihardlimit = dqg.dqb_ihardlimit;
+        dq->dqb_isoftlimit = dqg.dqb_isoftlimit;
+        dq->dqb_curinodes = dqg.dqb_curinodes;
+        dq->dqb_btime = dqg.dqb_btime;
+        dq->dqb_itime = dqg.dqb_itime;
+        dq->bsize = dqg.bsize;
+#else
         dq->dqb_btimelimit = dqg.dqb_btimelimit;
+#endif
     } /* if */
 
-
 #endif /* __svr4__ */
 
     return AFP_OK;
 }
 
 
-static int getquota( struct vol *vol, struct dqblk *dq, const u_int32_t bsize)
+static int getquota(const AFPObj *obj, struct vol *vol, struct dqblk *dq, const uint32_t bsize)
 {
     char *p;
 
@@ -658,8 +649,8 @@ static int getquota( struct vol *vol, struct dqblk *dq, const u_int32_t bsize)
     }
 #endif
 
-    return vol->v_nfs ? getnfsquota(vol, uuid, bsize, dq) :
-           getfsquota(vol, uuid, dq);
+    return vol->v_nfs ? getnfsquota(vol, obj->uid, bsize, dq) :
+      getfsquota(obj, vol, obj->uid, dq);
 }
 
 static int overquota( struct dqblk *dqblk)
@@ -707,41 +698,25 @@ static int overquota( struct dqblk *dqblk)
    work */
 #ifdef HAVE_2ARG_DBTOB
 #define tobytes(a, b)  dbtob((VolSpace) (a), (VolSpace) (b))
-#else 
+#else
 #define tobytes(a, b)  dbtob((VolSpace) (a))
 #endif
 
-int uquota_getvolspace( struct vol *vol, VolSpace *bfree, VolSpace *btotal, const u_int32_t bsize)
+int uquota_getvolspace(const AFPObj *obj, struct vol *vol, VolSpace *bfree, VolSpace *btotal, const uint32_t bsize)
 {
-	u_int64_t this_bsize;
+	uint64_t this_bsize;
 	struct dqblk dqblk;
 
 	this_bsize = bsize;
 			
-	if (getquota( vol, &dqblk, bsize) != 0 ) {
+	if (getquota(obj, vol, &dqblk, bsize) != 0 ) {
 		return( AFPERR_PARAM );
 	}
 
-#ifdef linux
+#ifdef __linux__
 	this_bsize = dqblk.bsize;
 #endif
 
-#ifdef DEBUG_QUOTA
-        LOG(log_debug, logtype_afpd, "after calling getquota in uquota_getvolspace!" );
-        LOG(log_debug, logtype_afpd, "dqb_ihardlimit: %u", dqblk.dqb_ihardlimit );
-        LOG(log_debug, logtype_afpd, "dqb_isoftlimit: %u", dqblk.dqb_isoftlimit );
-        LOG(log_debug, logtype_afpd, "dqb_curinodes : %u", dqblk.dqb_curinodes );
-        LOG(log_debug, logtype_afpd, "dqb_bhardlimit: %u", dqblk.dqb_bhardlimit );
-        LOG(log_debug, logtype_afpd, "dqb_bsoftlimit: %u", dqblk.dqb_bsoftlimit );
-        LOG(log_debug, logtype_afpd, "dqb_curblocks : %u", dqblk.dqb_curblocks );
-        LOG(log_debug, logtype_afpd, "dqb_btime     : %u", dqblk.dqb_btime );
-        LOG(log_debug, logtype_afpd, "dqb_itime     : %u", dqblk.dqb_itime );
-        LOG(log_debug, logtype_afpd, "bsize/this_bsize : %u/%u", bsize, this_bsize );
-	LOG(log_debug, logtype_afpd, "dqblk.dqb_bhardlimit size: %u", tobytes( dqblk.dqb_bhardlimit, this_bsize ));
-	LOG(log_debug, logtype_afpd, "dqblk.dqb_bsoftlimit size: %u", tobytes( dqblk.dqb_bsoftlimit, this_bsize ));
-	LOG(log_debug, logtype_afpd, "dqblk.dqb_curblocks  size: %u", tobytes( dqblk.dqb_curblocks, this_bsize ));
-#endif /* DEBUG_QUOTA */ 
-
 	/* no limit set for this user. it might be set in the future. */
 	if (dqblk.dqb_bsoftlimit == 0 && dqblk.dqb_bhardlimit == 0) {
         	*btotal = *bfree = ~((VolSpace) 0);
@@ -761,13 +736,6 @@ int uquota_getvolspace( struct vol *vol, VolSpace *bfree, VolSpace *btotal, cons
                  	  tobytes( dqblk.dqb_curblocks, this_bsize );
     	}
 
-#ifdef DEBUG_QUOTA
-        LOG(log_debug, logtype_afpd, "bfree          : %u", *bfree );
-        LOG(log_debug, logtype_afpd, "btotal         : %u", *btotal );
-        LOG(log_debug, logtype_afpd, "bfree          : %uKB", *bfree/1024 );
-        LOG(log_debug, logtype_afpd, "btotal         : %uKB", *btotal/1024 );
-#endif
-
 	return( AFP_OK );
 }
 #endif /* HAVE_LIBQUOTA */
