@@ -7,36 +7,55 @@
 #include "config.h"
 #endif /* HAVE_CONFIG_H */
 
+#include <arpa/inet.h>
+#include <ctype.h>
+#include <fcntl.h>
+
+#ifdef HAVE_KERBEROS
+#ifdef HAVE_KRB5_KRB5_H
+#include <krb5/krb5.h>
+#else
+#include <krb5.h>
+#endif /* HAVE_KRB5_KRB5_H */
+#endif /* HAVE_KERBEROS */
+
+#include <stdbool.h>
 #include <stdio.h>
 #include <stdlib.h>
-#include <unistd.h>
-#include <ctype.h>
 #include <string.h>
-#include <time.h>
-#include <sys/types.h>
-#include <sys/stat.h>
-#include <fcntl.h>
 #include <sys/socket.h>
-#include <atalk/logger.h>
+#include <sys/stat.h>
+#include <sys/types.h>
+#include <time.h>
+#include <unistd.h>
 
-#include <netatalk/at.h>
-#include <netatalk/endian.h>
 #include <atalk/dsi.h>
-#include <atalk/atp.h>
-#include <atalk/asp.h>
-#include <atalk/nbp.h>
+#include <atalk/globals.h>
+#include <atalk/logger.h>
 #include <atalk/unicode.h>
 #include <atalk/util.h>
-#include <atalk/globals.h>
 
-#include "status.h"
 #include "afp_config.h"
 #include "icon.h"
+#include "status.h"
+#include "uam_auth.h"
 
 static   size_t maxstatuslen = 0;
 
-static void status_flags(char *data, const int notif, const int ipok,
-                         const unsigned char passwdbits, const int dirsrvcs _U_, int flags)
+static int uam_gss_enabled(void)
+{
+    /* XXX: must be a better way to find out if uam_gss is active */
+    return auth_uamfind(UAM_SERVER_LOGIN_EXT,
+                        "Client Krb v2",
+                        sizeof("Client Krb v2")) != NULL;
+}
+
+static void status_flags(char *data,
+                         const int notif,
+                         const int ipok,
+                         const unsigned char passwdbits,
+                         const int dirsrvcs,
+                         int flags)
 {
     uint16_t           status;
 
@@ -44,7 +63,6 @@ static void status_flags(char *data, const int notif, const int ipok,
            | AFPSRVRINFO_SRVSIGNATURE
            | AFPSRVRINFO_SRVMSGS
            | AFPSRVRINFO_FASTBOZO
-           | AFPSRVRINFO_SRVRDIR
            | AFPSRVRINFO_SRVUTF8
            | AFPSRVRINFO_EXTSLEEP;
 
@@ -52,10 +70,12 @@ static void status_flags(char *data, const int notif, const int ipok,
         status |= AFPSRVRINFO_PASSWD;
     if (passwdbits & PASSWD_NOSAVE)
         status |= AFPSRVRINFO_NOSAVEPASSWD;
-    if (ipok) /* only advertise tcp/ip if we have a valid address */        
+    if (ipok) /* only advertise tcp/ip if we have a valid address */
         status |= AFPSRVRINFO_TCPIP;
-    if (notif) /* Default is yes */        
+    if (notif) /* Default is yes */
         status |= AFPSRVRINFO_SRVNOTIFY;
+    if (dirsrvcs)
+        status |= AFPSRVRINFO_SRVRDIR;
     if (flags & OPTION_UUID)
         status |= AFPSRVRINFO_UUID;
 
@@ -66,9 +86,9 @@ static void status_flags(char *data, const int notif, const int ipok,
 static int status_server(char *data, const char *server, const struct afp_options *options)
 {
     char                *start = data;
-    char                *Obj, *Type, *Zone;
+    char                *Obj;
     char		buf[32];
-    u_int16_t           status;
+    uint16_t           status;
     size_t		len;
 
     /* make room for all offsets before server name */
@@ -76,11 +96,8 @@ static int status_server(char *data, const char *server, const struct afp_option
 
     /* extract the obj part of the server */
     Obj = (char *) server;
-#ifndef NO_DDP
-    nbp_name(server, &Obj, &Type, &Zone);
-#endif
-    if ((size_t)-1 == (len = convert_string( 
-                           options->unixcharset, options->maccharset, 
+    if ((size_t)-1 == (len = convert_string(
+                           options->unixcharset, options->maccharset,
                            Obj, -1, buf, sizeof(buf))) ) {
         len = MIN(strlen(Obj), 31);
     	*data++ = len;
@@ -102,7 +119,7 @@ static int status_server(char *data, const char *server, const struct afp_option
      * signature and net address offsets if they're not going to be
      * used. as there are no offsets after them, it doesn't hurt to
      * have them specified though. so, we just do that to simplify
-     * things.  
+     * things.
      *
      * NOTE2: AFP3.1 Documentation states that the directory names offset
      * is a required feature, even though it can be set to zero.
@@ -116,7 +133,7 @@ static int status_server(char *data, const char *server, const struct afp_option
 static void status_machine(char *data)
 {
     char                *start = data;
-    u_int16_t           status;
+    uint16_t           status;
     int			len;
     const char		*machine = "Netatalk%s";
     char buf[AFPSTATUS_MACHLEN+1];
@@ -147,11 +164,11 @@ static void status_machine(char *data)
 }
 
 /* server signature is a 16-byte quantity */
-static u_int16_t status_signature(char *data, int *servoffset,
+static uint16_t status_signature(char *data, int *servoffset,
                                   const struct afp_options *options)
 {
     char                 *status;
-    u_int16_t            offset, sigoff;
+    uint16_t            offset, sigoff;
 
     status = data;
 
@@ -174,14 +191,11 @@ static u_int16_t status_signature(char *data, int *servoffset,
 }
 
 static size_t status_netaddress(char *data, int *servoffset,
-#ifndef NO_DDP
-                                const ASP asp,
-#endif
-                                const DSI *dsi,
-                                const struct afp_options *options)
+                             const DSI *dsi,
+                             const struct afp_options *options)
 {
     char               *begin;
-    u_int16_t          offset;
+    uint16_t          offset;
     size_t             addresses_len = 0;
 
     begin = data;
@@ -191,10 +205,10 @@ static size_t status_netaddress(char *data, int *servoffset,
     data += ntohs(offset);
 
     /* format:
-       Address count (byte) 
-       len (byte = sizeof(length + address type + address) 
+       Address count (byte)
+       len (byte = sizeof(length + address type + address)
        address type (byte, ip address = 0x01, ip + port = 0x02,
-                           ddp address = 0x03, fqdn = 0x04) 
+                           ddp address = 0x03, fqdn = 0x04)
        address (up to 254 bytes, ip = 4, ip + port = 6, ddp = 4)
        */
 
@@ -202,9 +216,6 @@ static size_t status_netaddress(char *data, int *servoffset,
        connection, but we don't have the ip address. to get around this,
        we turn off the status flag for tcp/ip. */
     *data++ = ((options->fqdn && dsi)? 1 : 0) + (dsi ? 1 : 0) +
-#ifndef NO_DDP
-        (asp ? 1 : 0) +
-#endif
               (((options->flags & OPTION_ANNOUNCESSH) && options->fqdn && dsi)? 1 : 0);
 
     /* ip address */
@@ -262,10 +273,10 @@ static size_t status_netaddress(char *data, int *servoffset,
             addresses_len += len+2;
         }
 
-        /* Annouce support for SSH tunneled AFP session, 
+        /* Annouce support for SSH tunneled AFP session,
          * this feature is available since 10.3.2.
          * According to the specs (AFP 3.1 p.225) this should
-         * be an IP+Port style value, but it only works with 
+         * be an IP+Port style value, but it only works with
          * a FQDN. OSX Server uses FQDN as well.
          */
         if ( len + 2 + addresses_len < maxstatuslen - offset) {
@@ -278,25 +289,8 @@ static size_t status_netaddress(char *data, int *servoffset,
         }
     }
 
-#ifndef NO_DDP
-    if (asp) {
-        const struct sockaddr_at *ddpaddr = atp_sockaddr(asp->asp_atp);
-
-        /* ddp address */
-        *data++ = 6;
-        *data++ = 0x03; /* ddp address */
-        memcpy(data, &ddpaddr->sat_addr.s_net, sizeof(ddpaddr->sat_addr.s_net));
-        data += sizeof(ddpaddr->sat_addr.s_net);
-        memcpy(data, &ddpaddr->sat_addr.s_node,
-               sizeof(ddpaddr->sat_addr.s_node));
-        data += sizeof(ddpaddr->sat_addr.s_node);
-        memcpy(data, &ddpaddr->sat_port, sizeof(ddpaddr->sat_port));
-        data += sizeof(ddpaddr->sat_port);
-    }
-#endif /* ! NO_DDP */
-
     /* calculate/store Directory Services Names offset */
-    offset = htons(data - begin); 
+    offset = htons(data - begin);
     *servoffset += sizeof(offset);
     memcpy(begin + *servoffset, &offset, sizeof(offset));
 
@@ -304,54 +298,37 @@ static size_t status_netaddress(char *data, int *servoffset,
     return (data - begin);
 }
 
-static size_t status_directorynames(char *data, int *diroffset, 
-				 const DSI *dsi _U_, 
-				 const struct afp_options *options)
+
+/**
+ * DirectoryNamesCount offset: uint16_t
+ * ...
+ * DirectoryNamesCount: uint8_t
+ * DirectoryNames: list of UTF-8 Pascal strings (uint8_t + char[1,255])
+ */
+static size_t status_directorynames(char *data,
+                                    int *diroffset,
+                                    const DSI *dsi _U_,
+                                    const struct afp_options *options)
 {
     char *begin = data;
-    u_int16_t offset;
-    memcpy(&offset, data + *diroffset, sizeof(offset));
+    uint16_t offset;
+
+    memcpy(&offset, begin + *diroffset, sizeof(offset));
     offset = ntohs(offset);
     data += offset;
 
-    /* I can not find documentation of any other purpose for the
-     * DirectoryNames field.
-     */
-    /*
-     * Try to synthesize a principal:
-     * service '/' fqdn '@' realm
-     */
-    if (options->k5service && options->k5realm && options->fqdn) {
-	/* should k5princ be utf8 encoded? */
-	size_t len;
-	char *p = strchr( options->fqdn, ':' );
-	if (p) 
-	    *p = '\0';
-	len = strlen( options->k5service ) 
-			+ strlen( options->fqdn )
-			+ strlen( options->k5realm );
-	len+=2; /* '/' and '@' */
-	if ( len > 255 || len+2 > maxstatuslen - offset) {
-	    *data++ = 0;
-	    LOG ( log_error, logtype_afpd, "status: could not set directory service list, no more room");
-	}	 
-	else {
-	    *data++ = 1; /* DirectoryNamesCount */
-	    *data++ = len;
-	    snprintf( data, len + 1, "%s/%s@%s", options->k5service,
-				options->fqdn, options->k5realm );
-	    data += len;
-	    if (p)
-	        *p = ':';
-       }
+    if (!uam_gss_enabled() || !options->k5principal) {
+        *data = 0;
+        data++;
     } else {
-	*data++ = 0;
+        memcpy(data, options->k5principal, options->k5principal_buflen);
+        data += options->k5principal_buflen;
     }
 
     /* Calculate and store offset for UTF8ServerName */
-    *diroffset += sizeof(u_int16_t);
+    *diroffset += sizeof(uint16_t);
     offset = htons(data - begin);
-    memcpy(begin + *diroffset, &offset, sizeof(u_int16_t));
+    memcpy(begin + *diroffset, &offset, sizeof(uint16_t));
 
     /* return length of buffer */
     return (data - begin);
@@ -361,45 +338,35 @@ static size_t status_utf8servername(char *data, int *nameoffset,
 				 const DSI *dsi _U_,
 				 const struct afp_options *options)
 {
-    char *Obj, *Type, *Zone;
-    u_int16_t namelen;
+    uint16_t namelen;
     size_t len;
     char *begin = data;
-    u_int16_t offset;
+    uint16_t offset;
 
     memcpy(&offset, data + *nameoffset, sizeof(offset));
     offset = ntohs(offset);
     data += offset;
 
-    /* FIXME:
-     * What is the valid character range for an nbpname?
-     *
-     * Apple's server likes to use the non-qualified hostname
-     * This obviously won't work very well if multiple servers are running
-     * on the box.
-     */
+    LOG(log_info, logtype_afpd, "servername: %s", options->hostname);
 
-    /* extract the obj part of the server */
-    Obj = (char *) (options->server ? options->server : options->hostname);
-#ifndef NO_DDP
-    nbp_name(options->server ? options->server : options->hostname, &Obj, &Type, &Zone);
-#endif
-    if ((size_t) -1 == (len = convert_string (
-					options->unixcharset, CH_UTF8_MAC, 
-					Obj, -1, data+sizeof(namelen), maxstatuslen-offset )) ) {
-	LOG ( log_error, logtype_afpd, "Could not set utf8 servername");
+    if ((len = convert_string(options->unixcharset,
+                              CH_UTF8_MAC,
+                              options->hostname,
+                              -1,
+                              data + sizeof(namelen),
+                              maxstatuslen-offset)) == (size_t)-1) {
+        LOG(log_error, logtype_afpd, "Could not set utf8 servername");
 
-	/* set offset to 0 */
-	memset(begin + *nameoffset, 0, sizeof(offset));
+        /* set offset to 0 */
+        memset(begin + *nameoffset, 0, sizeof(offset));
         data = begin + offset;
-    }
-    else {
+    } else {
     	namelen = htons(len);
     	memcpy( data, &namelen, sizeof(namelen));
     	data += sizeof(namelen);
     	data += len;
     	offset = htons(offset);
-    	memcpy(begin + *nameoffset, &offset, sizeof(u_int16_t));
+    	memcpy(begin + *nameoffset, &offset, sizeof(uint16_t));
     }
 
     /* return length of buffer */
@@ -413,7 +380,7 @@ static void status_icon(char *data, const unsigned char *icondata,
 {
     char                *start = data;
     char                *sigdata = data + sigoffset;
-    u_int16_t		ret, status;
+    uint16_t		ret, status;
 
     memcpy(&status, start + AFPSTATUS_ICONOFF, sizeof(status));
     if ( icondata == NULL ) {
@@ -433,48 +400,27 @@ static void status_icon(char *data, const unsigned char *icondata,
 
 /* ---------------------
  */
-void status_init(AFPConfig *aspconfig, AFPConfig *dsiconfig,
-                 const struct afp_options *options)
+void status_init(AFPObj *obj, DSI *dsi)
 {
-#ifndef NO_DDP
-    ASP asp;
-#endif
-    DSI *dsi;
-    char *status = NULL;
+    char *status = dsi->status;
     size_t statuslen;
-    int c, sigoff, ipok;
+    int c, sigoff, ipok = 0;
+    const struct afp_options *options = &obj->options;
 
-    if (!(aspconfig || dsiconfig) || !options)
-        return;
+    maxstatuslen = sizeof(dsi->status);
 
-#ifndef NO_DDP
-    if (aspconfig) {
-        status = aspconfig->status;
-        maxstatuslen=sizeof(aspconfig->status);
-        asp = aspconfig->obj.handle;
-    } else
-        asp = NULL;
-#endif
-	
-    ipok = 0;
-    if (dsiconfig) {
-        status = dsiconfig->status;
-        maxstatuslen=sizeof(dsiconfig->status);
-        dsi = dsiconfig->obj.handle;
-        if (dsi->server.ss_family == AF_INET) { /* IPv4 */
-            const struct sockaddr_in *sa4 = (struct sockaddr_in *)&dsi->server;
-            ipok = sa4->sin_addr.s_addr ? 1 : 0;
-        } else { /* IPv6 */
-            const struct sockaddr_in6 *sa6 = (struct sockaddr_in6 *)&dsi->server;
-            for (int i=0; i<16; i++) {
-                if (sa6->sin6_addr.s6_addr[i]) {
-                    ipok = 1;
-                    break;
-                }
+    if (dsi->server.ss_family == AF_INET) { /* IPv4 */
+        const struct sockaddr_in *sa4 = (struct sockaddr_in *)&dsi->server;
+        ipok = sa4->sin_addr.s_addr ? 1 : 0;
+    } else { /* IPv6 */
+        const struct sockaddr_in6 *sa6 = (struct sockaddr_in6 *)&dsi->server;
+        for (int i=0; i<16; i++) {
+            if (sa6->sin6_addr.s6_addr[i]) {
+                ipok = 1;
+                break;
             }
         }
-    } else
-        dsi = NULL;
+    }
 
     /*
      * These routines must be called in order -- earlier calls
@@ -493,64 +439,36 @@ void status_init(AFPConfig *aspconfig, AFPConfig *dsiconfig,
      *
      * at the appropriate offsets:
      * machine type, afp versions, uams, server signature
-     * (16-bytes), network addresses, volume icon/mask 
+     * (16-bytes), network addresses, volume icon/mask
      */
 
     status_flags(status,
-                 options->server_notif,
+                 options->flags & OPTION_SERVERNOTIF,
                  (options->fqdn || ipok),
-                 options->passwdbits, 
-                 (options->k5service && options->k5realm && options->fqdn),
+                 options->passwdbits,
+                 1,
                  options->flags);
     /* returns offset to signature offset */
-    c = status_server(status, options->server ? options->server :
-                      options->hostname, options);
+    c = status_server(status, options->hostname, options);
     status_machine(status);
-    status_versions(status,
-#ifndef NO_DDP
-                    asp,
-#endif
-                    dsi);
+    status_versions(status, dsi);
     status_uams(status, options->uamlist);
-    if (options->flags & OPTION_CUSTOMICON)
-        status_icon(status, icon, sizeof(icon), c);
-    else
-        status_icon(status, apple_atalk_icon, sizeof(apple_atalk_icon), c);
+    status_icon(status, icon, sizeof(icon), c);
 
     sigoff = status_signature(status, &c, options);
     /* c now contains the offset where the netaddress offset lives */
 
-    status_netaddress(status, &c,
-#ifndef NO_DDP
-                      asp,
-#endif
-                      dsi, options);
+    status_netaddress(status, &c, dsi, options);
     /* c now contains the offset where the Directory Names Count offset lives */
 
     statuslen = status_directorynames(status, &c, dsi, options);
     /* c now contains the offset where the UTF-8 ServerName offset lives */
 
-    if ( statuslen < maxstatuslen) 
+    if ( statuslen < maxstatuslen)
         statuslen = status_utf8servername(status, &c, dsi, options);
 
-#ifndef NO_DDP
-    if (aspconfig) {
-        if (dsiconfig) /* status is dsiconfig->status */
-            memcpy(aspconfig->status, status, statuslen);
-        asp_setstatus(asp, status, statuslen);
-        aspconfig->signature = status + sigoff;
-        aspconfig->statuslen = statuslen;
-    }
-#endif /* ! NO_DDP */
-
-    if (dsiconfig) {
-        if ((options->flags & OPTION_CUSTOMICON) == 0) {
-            status_icon(status, apple_tcp_icon, sizeof(apple_tcp_icon), 0);
-        }
-        dsi_setstatus(dsi, status, statuslen);
-        dsiconfig->signature = status + sigoff;
-        dsiconfig->statuslen = statuslen;
-    }
+    dsi->signature = status + sigoff;
+    dsi->statuslen = statuslen;
 }
 
 /* set_signature()                                                    */
@@ -559,10 +477,9 @@ void status_init(AFPConfig *aspconfig, AFPConfig *dsiconfig,
 /* If not found in conf file, genarate and append in conf file.       */
 /* If conf file don't exist, create and genarate.                     */
 /* If cannot open conf file, use one-time signature.                  */
-/* If -signature user:xxxxx, use it.                                  */
+/* If signature = xxxxx, use it.                                      */
 
 void set_signature(struct afp_options *options) {
-    char *usersign;
     int fd, i;
     struct stat tmpstat;
     char *servername_conf;
@@ -571,49 +488,37 @@ void set_signature(struct afp_options *options) {
     FILE *fp = NULL;
     size_t len;
     char *server_tmp;
-    
-    server_tmp = (options->server ? options->server : options->hostname);
-    if (strcmp(options->signatureopt, "auto") == 0) {
-        goto server_signature_auto;   /* default */
-    } else if (strcmp(options->signatureopt, "host") == 0) {
-        LOG(log_warning, logtype_afpd, "WARNING: option \"-signature host\" is obsoleted. Switching back to auto.", options->signatureopt);
-        goto server_signature_auto;   /* same as auto */
-    } else if (strncmp(options->signatureopt, "user", 4) == 0) {
-        goto server_signature_user;   /*  user string */
-    } else {
-        LOG(log_error, logtype_afpd, "ERROR: option \"-signature %s\" is not valid. Switching back to auto.", options->signatureopt);
-        goto server_signature_auto;   /* switch back to auto*/
-    }
-    
-server_signature_user:
-    
-    /* Signature type is user string */
+
+    server_tmp = options->hostname;
     len = strlen(options->signatureopt);
-    if (len <= 5) {
-        LOG(log_warning, logtype_afpd, "WARNING: option \"-signature %s\" is not valid. Switching back to auto.", options->signatureopt);
-        goto server_signature_auto;
-    }
-    usersign = options->signatureopt + 5;
-    len = len - 5;
-    if (len > 16) {
-        LOG(log_warning, logtype_afpd, "WARNING: signature user string %s is very long !",  usersign);
+    if (len == 0) {
+        goto server_signature_auto;   /* default */
+    } else if (len < 3) {
+        LOG(log_warning, logtype_afpd, "WARNING: signature string %s is very short !", options->signatureopt);
+        goto server_signature_user;
+    } else if (len > 16) {
+        LOG(log_warning, logtype_afpd, "WARNING: signature string %s is very long !", options->signatureopt);
         len = 16;
-    } else if (len >= 3) {
-        LOG(log_info, logtype_afpd, "signature user string is %s.", usersign);
+        goto server_signature_user;
     } else {
-        LOG(log_warning, logtype_afpd, "WARNING: signature user string %s is very short !",  usersign);
+        LOG(log_info, logtype_afpd, "signature string is %s.", options->signatureopt);
+        goto server_signature_user;
     }
+
+server_signature_user:
+
+    /* Signature is defined in afp.conf */
     memset(options->signature, 0, 16);
-    memcpy(options->signature, usersign, len);
+    memcpy(options->signature, options->signatureopt, len);
     goto server_signature_done;
-    
+
 server_signature_auto:
-    
+
     /* Signature type is auto, using afp_signature.conf */
     if (!stat(options->sigconffile, &tmpstat)) {                /* conf file exists? */
         if ((fp = fopen(options->sigconffile, "r")) != NULL) {  /* read open? */
             /* scan in the conf file */
-            while (fgets(buf, sizeof(buf), fp) != NULL) { 
+            while (fgets(buf, sizeof(buf), fp) != NULL) {
                 p = buf;
                 while (p && isblank(*p))
                     p++;
@@ -631,10 +536,10 @@ server_signature_auto:
                 p++;
                 if (*p == '\0')
                     continue;                             /* syntax error: missing signature */
-                
+
                 if (strcmp(server_tmp, servername_conf))
                     continue;                             /* another servername */
-                
+
                 while (p && isblank(*p))
                     p++;
                 if ( 16 == sscanf(p, "%2hhX%2hhX%2hhX%2hhX%2hhX%2hhX%2hhX%2hhX%2hhX%2hhX%2hhX%2hhX%2hhX%2hhX%2hhX%2hhX",
@@ -658,14 +563,9 @@ server_signature_auto:
                     goto server_signature_random;
                 } else {
                     fseek(fp, -1L, SEEK_END);
-                    if(fgetc(fp) != '\n')
-                    {
-                        /* Required -  C11 7.21.5.3p7 */
-                        fseek(fp, 0L, SEEK_END);
-                        fputc('\n', fp); /* last char is \n? */
-                    }
+                    if(fgetc(fp) != '\n') fputc('\n', fp); /* last char is \n? */
                     goto server_signature_random;
-                }                    
+                }
             } else {
                 LOG(log_error, logtype_afpd, "ERROR: Cannot write in %s (%s). Using one-time signature.",
                     options->sigconffile, strerror(errno));
@@ -691,9 +591,9 @@ server_signature_auto:
         header = 1;
         goto server_signature_random;
     }
-    
+
 server_signature_random:
-    
+
     /* generate signature from random number */
     randombytes(options->signature, 16);
 
@@ -704,9 +604,9 @@ server_signature_random:
         fprintf(fp, "# ServerSignature is unique identifier used to prevent logging on to\n");
         fprintf(fp, "# the same server twice.\n");
         fprintf(fp, "# \n");
-        fprintf(fp, "# If setting \"-signature user:xxxxx\" in afpd.conf, this file is not used.\n\n");
+        fprintf(fp, "# If setting \"signature = xxxxx\" in afp.conf, this file is not used.\n\n");
     }
-    
+
     if (fp) {
         fprintf(fp, "\"%s\"\t", server_tmp);
         for (i=0 ; i<16 ; i++) {
@@ -715,13 +615,12 @@ server_signature_random:
         fprintf(fp, "%s", "\n");
         fclose(fp);
     }
-    
+
 server_signature_done:
-    
+
     /* retrun */
     LOG(log_info, logtype_afpd,
-        "\"%s\"'s signature is %02X%02X%02X%02X%02X%02X%02X%02X%02X%02X%02X%02X%02X%02X%02X%02X",
-        server_tmp,
+        "signature is %02X%02X%02X%02X%02X%02X%02X%02X%02X%02X%02X%02X%02X%02X%02X%02X",
         (options->signature)[ 0], (options->signature)[ 1],
         (options->signature)[ 2], (options->signature)[ 3],
         (options->signature)[ 4], (options->signature)[ 5],
@@ -730,16 +629,15 @@ server_signature_done:
         (options->signature)[10], (options->signature)[11],
         (options->signature)[12], (options->signature)[13],
         (options->signature)[14], (options->signature)[15]);
-    
+
     return;
 }
 
 /* this is the same as asp/dsi_getstatus */
 int afp_getsrvrinfo(AFPObj *obj, char *ibuf _U_, size_t ibuflen _U_, char *rbuf, size_t *rbuflen)
 {
-    AFPConfig *config = obj->config;
+    memcpy(rbuf, obj->dsi->status, obj->dsi->statuslen);
+    *rbuflen = obj->dsi->statuslen;
 
-    memcpy(rbuf, config->status, config->statuslen);
-    *rbuflen = config->statuslen;
     return AFP_OK;
 }
