@@ -1,9 +1,7 @@
 #ifndef AFPD_STATUS_H
 #define AFPD_STATUS_H 1
 
-#include <sys/types.h>
 #include <atalk/dsi.h>
-#include <atalk/asp.h>
 #include <atalk/globals.h>
 
 #include "afp_config.h"
@@ -15,7 +13,7 @@
 #define AFPSTATUS_ICONOFF     6
 #define AFPSTATUS_FLAGOFF     8
 
-/* AFPSTATUS_PRELEN is the number of bytes for status data prior to 
+/* AFPSTATUS_PRELEN is the number of bytes for status data prior to
  * the ServerName field.
  *
  * This is two bytes of offset space for the MachineType, AFPVersionCount,
@@ -35,20 +33,9 @@
 /* AFPSTATUS_MACHLEN is the number of characters for the MachineType. */
 #define AFPSTATUS_MACHLEN     16
 
-
-#define PASSWD_NONE     0
-#define PASSWD_SET     (1 << 0)
-#define PASSWD_NOSAVE  (1 << 1)
-#define PASSWD_ALL     (PASSWD_SET | PASSWD_NOSAVE)
-
-extern void status_versions (char * /*status*/,
-#ifndef NO_DDP
-                             const ASP,
-#endif
-                             const DSI *);
+extern void status_versions (char * /*status*/, const DSI *);
 extern void status_uams (char * /*status*/, const char * /*authlist*/);
-extern void status_init (AFPConfig *, AFPConfig *,
-                                 const struct afp_options *);
+extern void status_init (AFPObj *, DSI *dsi);
 extern void set_signature(struct afp_options *);
 
 /* FP functions */
