@@ -1,10 +1,8 @@
 #ifndef AFPD_UNIX_H
 #define AFPD_UNIX_H
 
-#ifdef HAVE_SYS_CDEFS_H
-#include <sys/types.h>
-#endif /* HAVE_SYS_CDEFS_H */
-#include <netatalk/endian.h>
+#include <arpa/inet.h>
+
 #include "config.h"
 #include "volume.h"
 
@@ -12,7 +10,7 @@
 #include <sys/vfs.h>
 #endif /* HAVE_SYS_VFS_H */
 
-#if defined(HAVE_STATFS_H) 
+#if defined(HAVE_STATFS_H)
 #include <sys/statfs.h>
 /* this might not be right. */
 #define f_mntfromname f_fname
@@ -29,14 +27,17 @@
 #include <sys/mnttab.h>
 #endif /* __svr4__ || HAVE_SYS_MNTTAB_H */
 
-#if defined(HAVE_SYS_MOUNT_H)
+#if defined(__DragonFly__)
+#define dqblk ufs_dqblk
+#endif
+
+#if defined(HAVE_SYS_MOUNT_H) || defined(BSD4_4) || defined(__linux__)
 #include <sys/mount.h>
-#endif /* HAVE_SYS_MOUNT_H */
+#endif /* HAVE_SYS_MOUNT_H || BSD4_4 || __linux__ */
 
-#if defined(HAVE_MNTENT_H)
+#if defined(__linux__) || defined(HAVE_MNTENT_H)
 #include <mntent.h>
-#endif /* HAVE_MNTENT_H */
-
+#endif /* linux || HAVE_MNTENT_H */
 
 #ifndef NO_QUOTA_SUPPORT
 #if !defined(HAVE_LIBQUOTA)
@@ -45,25 +46,29 @@
 #define dqb_btimelimit  dqb_btime
 #endif /* ! __svr4__ || HAVE_DQB_BTIMELIMIT */
 
-#if defined(linux) || defined(HAVE_QUOTA_H)
-#ifndef NEED_QUOTACTL_WRAPPER
-/*#include <sys/quota.h>*/
-/*long quotactl (int, const char *, unsigned int, caddr_t); */
-/* extern long quotactl (int, const char *, long, caddr_t); */
-
-#else /* ! NEED_QUOTACTL_WRAPPER */
+#if defined(__linux__) || defined(HAVE_QUOTA_H)
 #include <asm/types.h>
 #include <asm/unistd.h>
 #include <linux/quota.h>
-#endif /* ! NEED_QUOTACTL_WRAPPER */
 #endif /* linux || HAVE_QUOTA_H */
 
-#ifdef __svr4__ 
+#ifdef __svr4__
 #include <sys/fs/ufs_quota.h>
 #endif /* __svr4__ */
 
 #ifdef BSD4_4
+#if defined(__DragonFly__)
+#include <vfs/ufs/quota.h>
+#elif defined(__NetBSD__)
 #include <ufs/ufs/quota.h>
+#include <ufs/ufs/quota1.h>
+#include <ufs/ufs/quota2.h>
+#elif defined(__APPLE__)
+#include <sys/quota.h>
+#define dqb_curblocks dqb_curbytes
+#else /* DragonFly */
+#include <ufs/ufs/quota.h>
+#endif /* DragonFly */
 #endif /* BSD4_4 */
 
 #ifdef HAVE_UFS_QUOTA_H
@@ -75,10 +80,7 @@
 #include <sys/stat.h>
 #include "directory.h"
 
-
-#if defined (linux)
-
-#define MAXQUOTAS 2
+#if defined (__linux__)
 
 /* definitions from sys/quota.h */
 #define USRQUOTA  0             /* element used for user quotas */
@@ -185,24 +187,25 @@ extern long quotactl (unsigned int, const char *, int, caddr_t);
 
 #endif /* linux */
 
-extern int getnfsquota (struct vol *, const int, const u_int32_t,
-                                struct dqblk *);
+extern int getnfsquota (struct vol *, const int, const uint32_t,
+                        struct dqblk *);
 
 #endif /* ! HAVE_LIBQUOTA */
-extern int uquota_getvolspace (struct vol *, VolSpace *, VolSpace *,
-                                       const u_int32_t);
+extern int uquota_getvolspace (const AFPObj *obj, struct vol *, VolSpace *, VolSpace *,
+                               const uint32_t);
 #endif /* NO_QUOTA_SUPPORT */
 
 extern struct afp_options default_options;
 
-extern int gmem            (const gid_t);
-extern int setdeskmode      (const mode_t);
-extern int setdirunixmode   (const struct vol *, const char *, mode_t);
+extern int setdirunixmode   (const struct vol *, char *, mode_t);
 extern int setdirmode       (const struct vol *, const char *, mode_t);
-extern int setdeskowner     (const uid_t, const gid_t);
 extern int setdirowner      (const struct vol *, const char *, const uid_t, const gid_t);
 extern int setfilunixmode   (const struct vol *, struct path*, const mode_t);
 extern int setfilowner      (const struct vol *, const uid_t, const gid_t, struct path*);
-extern void accessmode      (const struct vol *, char *, struct maccess *, struct dir *, struct stat *);
+extern void accessmode      (const AFPObj *obj, const struct vol *, char *, struct maccess *, struct dir *, struct stat *);
+
+#ifdef AFS	
+    #define accessmode afsmode
+#endif
 
 #endif /* UNIX_H */
