@@ -8,20 +8,21 @@
 #include "config.h"
 #endif /* HAVE_CONFIG_H */
 
-#include <stdio.h>
+#include <arpa/inet.h>
 #include <errno.h>
+#include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#include <sys/types.h>
 #include <sys/stat.h>
 #include <sys/types.h>
 #include <unistd.h>
 
 #include <db.h>
 
+#include <atalk/cnid.h>
+#include <atalk/errchk.h>
 #include <atalk/logger.h>
 #include <atalk/util.h>
-#include <atalk/errchk.h>
 
 #include "db_param.h"
 #include "dbif.h"
@@ -49,7 +50,7 @@ static int dbif_stamp(DBD *dbd, void *buffer, int size)
 
     /* chdir to db_envhome */
     if ((chdir(dbd->db_envhome)) != 0) {
-        LOG(log_error, logtype_cnid, "error chdiring to db_env '%s': %s", dbd->db_envhome, strerror(errno));        
+        LOG(log_error, logtype_cnid, "error chdiring to db_env '%s': %s", dbd->db_envhome, strerror(errno));
         EC_FAIL;
     }
 
@@ -66,7 +67,7 @@ static int dbif_stamp(DBD *dbd, void *buffer, int size)
 EC_CLEANUP:
     if (cwd != -1) {
         if (fchdir(cwd) != 0) {
-            LOG(log_error, logtype_cnid, "error chdiring back: %s", strerror(errno));        
+            LOG(log_error, logtype_cnid, "error chdiring back: %s", strerror(errno));
             EC_STATUS(-1);
         }
         close(cwd);
@@ -88,13 +89,11 @@ EC_CLEANUP:
 static int dbif_init_rootinfo(DBD *dbd, int version)
 {
     DBT key, data;
-    uint32_t v;
+    uint32_t uint32;
     char buf[ROOTINFO_DATALEN];
 
     LOG(log_debug, logtype_cnid, "Setting CNID database version to %u", version);
 
-    v = version;
-    v = htonl(v);
 
     memset(&key, 0, sizeof(key));
     memset(&data, 0, sizeof(data));
@@ -104,7 +103,13 @@ static int dbif_init_rootinfo(DBD *dbd, int version)
     data.size = ROOTINFO_DATALEN;
 
     memcpy(buf, ROOTINFO_DATA, ROOTINFO_DATALEN);
-    memcpy(buf + CNID_DID_OFS, &v, sizeof(v));
+
+    uint32 = htonl(CNID_START - 1);
+    memcpy(buf + CNID_TYPE_OFS, &uint32, sizeof(uint32));
+
+    uint32 = htonl(version);
+    memcpy(buf + CNID_DID_OFS, &uint32, sizeof(uint32));
+
     if (dbif_stamp(dbd, buf + CNID_DEV_OFS, CNID_DEV_LEN) < 0)
         return -1;
 
@@ -216,7 +221,7 @@ static int dbif_upgrade(DBD *dbd)
             return -1;
     }
 
-    /* 
+    /*
      * Do upgrade stuff ...
      */
 
@@ -249,7 +254,7 @@ static int dbif_openlog(DBD *dbd)
 
     /* chdir to db_envhome */
     if ((chdir(dbd->db_envhome)) != 0) {
-        LOG(log_error, logtype_cnid, "error chdiring to db_env '%s': %s", dbd->db_envhome, strerror(errno));        
+        LOG(log_error, logtype_cnid, "error chdiring to db_env '%s': %s", dbd->db_envhome, strerror(errno));
         ret = -1;
         goto exit;
     }
@@ -265,7 +270,7 @@ static int dbif_openlog(DBD *dbd)
 exit:
     if (cwd != -1) {
         if ((fchdir(cwd)) != 0) {
-            LOG(log_error, logtype_cnid, "error chdiring back: %s", strerror(errno));        
+            LOG(log_error, logtype_cnid, "error chdiring back: %s", strerror(errno));
             ret = -1;
         }
         close(cwd);
@@ -293,7 +298,7 @@ static int dbif_logautorem(DBD *dbd)
 
     /* chdir to db_envhome */
     if ((chdir(dbd->db_envhome)) != 0) {
-        LOG(log_error, logtype_cnid, "error chdiring to db_env '%s': %s", dbd->db_envhome, strerror(errno));        
+        LOG(log_error, logtype_cnid, "error chdiring to db_env '%s': %s", dbd->db_envhome, strerror(errno));
         ret = -1;
         goto exit;
     }
@@ -318,7 +323,7 @@ static int dbif_logautorem(DBD *dbd)
 exit:
     if (cwd != -1) {
         if ((fchdir(cwd)) != 0) {
-            LOG(log_error, logtype_cnid, "error chdiring back: %s", strerror(errno));        
+            LOG(log_error, logtype_cnid, "error chdiring back: %s", strerror(errno));
             ret = -1;
         }
         close(cwd);
@@ -326,91 +331,6 @@ exit:
     return ret;
 }
 
-/*!
- * Get lock on db lock file
- *
- * @args cmd       (r) lock command:
- *                     LOCK_FREE:   close lockfd
- *                     LOCK_UNLOCK: unlock lockm keep lockfd open
- *                     LOCK_EXCL:   F_WRLCK on lockfd
- *                     LOCK_SHRD:   F_RDLCK on lockfd
- * @args dbpath    (r) path to lockfile, only used on first call,
- *                     later the stored fd is used
- * @returns            LOCK_FREE/LOCK_UNLOCK return 0 on success, -1 on error
- *                     LOCK_EXCL/LOCK_SHRD return LOCK_EXCL or LOCK_SHRD respectively on
- *                     success, 0 if the lock couldn't be acquired, -1 on other errors
- */
-int get_lock(int cmd, const char *dbpath)
-{
-    static int lockfd = -1;
-    int ret;
-    char lockpath[PATH_MAX];
-    struct stat st;
-
-    switch (cmd) {
-    case LOCK_FREE:
-        if (lockfd == -1)
-            return -1;
-        close(lockfd);
-        lockfd = -1;
-        return 0;
-
-    case LOCK_UNLOCK:
-        if (lockfd == -1)
-            return -1;
-        return unlock(lockfd, 0, SEEK_SET, 0);
-
-    case LOCK_EXCL:
-    case LOCK_SHRD:
-        if (lockfd == -1) {
-            if ( (strlen(dbpath) + (strlen(LOCKFILENAME) + 1)) > (PATH_MAX - 1) ) {
-                LOG(log_error, logtype_cnid, ".AppleDB pathname too long");
-                return -1;
-            }
-            strncpy(lockpath, dbpath, PATH_MAX - 1);
-            strcat(lockpath, "/");
-            strcat(lockpath, LOCKFILENAME);
-
-            if ((lockfd = open(lockpath, O_RDWR | O_CREAT, 0644)) < 0) {
-                LOG(log_error, logtype_cnid, "Error opening lockfile: %s", strerror(errno));
-                return -1;
-            }
-
-            if ((stat(dbpath, &st)) != 0) {
-                LOG(log_error, logtype_cnid, "Error statting lockfile: %s", strerror(errno));
-                return -1;
-            }
-
-            if ((chown(lockpath, st.st_uid, st.st_gid)) != 0) {
-                LOG(log_error, logtype_cnid, "Error inheriting lockfile permissions: %s",
-                         strerror(errno));
-                return -1;
-            }
-        }
-    
-        if (cmd == LOCK_EXCL)
-            ret = write_lock(lockfd, 0, SEEK_SET, 0);
-        else
-            ret = read_lock(lockfd, 0, SEEK_SET, 0);
-
-        if (ret != 0) {
-            if (cmd == LOCK_SHRD)
-                LOG(log_error, logtype_cnid, "Volume CNID db is locked, try again...");
-            return 0; 
-        }
-
-        LOG(log_debug, logtype_cnid, "get_lock: got %s lock",
-            cmd == LOCK_EXCL ? "LOCK_EXCL" : "LOCK_SHRD");    
-        return cmd;
-
-    default:
-        return -1;
-    } /* switch(cmd) */
-
-    /* deadc0de, never get here */
-    return -1;
-}
-
 /* --------------- */
 DBD *dbif_init(const char *envhome, const char *filename)
 {
@@ -421,10 +341,8 @@ DBD *dbif_init(const char *envhome, const char *filename)
 
     /* filename == NULL means in memory db */
     if (filename) {
-        if (! envhome) {
-            free(dbd);
+        if (! envhome)
             return NULL;
-        }
 
         dbd->db_envhome = strdup(envhome);
         if (NULL == dbd->db_envhome) {
@@ -439,7 +357,7 @@ DBD *dbif_init(const char *envhome, const char *filename)
             return NULL;
         }
     }
-    
+
     dbd->db_table[DBIF_CNID].name        = "cnid2.db";
     dbd->db_table[DBIF_IDX_DEVINO].name  = "devino.db";
     dbd->db_table[DBIF_IDX_DIDNAME].name = "didname.db";
@@ -460,7 +378,7 @@ DBD *dbif_init(const char *envhome, const char *filename)
     return dbd;
 }
 
-/* 
+/*
    We must open the db_env with an absolute pathname, as `dbd` keeps chdir'ing, which
    breaks e.g. bdb logfile-rotation with relative pathnames.
    But still we use relative paths with DB_ERRLOGFILE
@@ -507,7 +425,7 @@ int dbif_env_open(DBD *dbd, struct db_param *dbp, uint32_t dbenv_oflags)
             dbd->db_env = NULL;
             return -1;
         }
-        dbd->db_errlog = NULL;        
+        dbd->db_errlog = NULL;
 
         if ((ret = db_env_create(&dbd->db_env, 0))) {
             LOG(log_error, logtype_cnid, "error creating DB environment after recovery: %s",
@@ -603,13 +521,13 @@ int dbif_open(DBD *dbd, struct db_param *dbp, int reindex)
             LOG(log_error, logtype_cnid, "error opening cwd: %s", strerror(errno));
             return -1;
         }
-        
+
         /* chdir to db_envhome. makes it easier checking for old db files and creating db_errlog file  */
         if ((chdir(dbd->db_envhome)) != 0) {
-            LOG(log_error, logtype_cnid, "error chdiring to db_env '%s': %s", dbd->db_envhome, strerror(errno));        
+            LOG(log_error, logtype_cnid, "error chdiring to db_env '%s': %s", dbd->db_envhome, strerror(errno));
             return -1;
         }
-        
+
         if ((stat(dbd->db_filename, &st)) == 0) {
             LOG(log_debug, logtype_cnid, "See if we can upgrade the CNID database...");
             if ((ret = db_create(&upgrade_db, dbd->db_env, 0))) {
@@ -630,9 +548,9 @@ int dbif_open(DBD *dbd, struct db_param *dbp, int reindex)
             }
             LOG(log_debug, logtype_cnid, "Finished BerkeleyBD upgrade check");
         }
-        
+
         if ((fchdir(cwd)) != 0) {
-            LOG(log_error, logtype_cnid, "error chdiring back: %s", strerror(errno));        
+            LOG(log_error, logtype_cnid, "error chdiring back: %s", strerror(errno));
             return -1;
         }
     }
@@ -694,7 +612,7 @@ int dbif_open(DBD *dbd, struct db_param *dbp, int reindex)
         LOG(log_info, logtype_cnid, "Reindexing did/name index...");
     if ((ret = dbd->db_table[0].db->associate(dbd->db_table[DBIF_CNID].db,
                                               dbd->db_txn,
-                                              dbd->db_table[DBIF_IDX_DIDNAME].db, 
+                                              dbd->db_table[DBIF_IDX_DIDNAME].db,
                                               didname,
                                               (reindex) ? DB_CREATE : 0))
          != 0) {
@@ -706,9 +624,9 @@ int dbif_open(DBD *dbd, struct db_param *dbp, int reindex)
 
     if (reindex)
         LOG(log_info, logtype_cnid, "Reindexing dev/ino index...");
-    if ((ret = dbd->db_table[0].db->associate(dbd->db_table[0].db, 
+    if ((ret = dbd->db_table[0].db->associate(dbd->db_table[0].db,
                                               dbd->db_txn,
-                                              dbd->db_table[DBIF_IDX_DEVINO].db, 
+                                              dbd->db_table[DBIF_IDX_DEVINO].db,
                                               devino,
                                               (reindex) ? DB_CREATE : 0))
         != 0) {
@@ -731,12 +649,12 @@ int dbif_open(DBD *dbd, struct db_param *dbp, int reindex)
             return -1;
     }
 
-    if ((ret = dbd->db_table[0].db->associate(dbd->db_table[0].db, 
+    if ((ret = dbd->db_table[0].db->associate(dbd->db_table[0].db,
                                               dbd->db_txn,
-                                              dbd->db_table[DBIF_IDX_NAME].db, 
+                                              dbd->db_table[DBIF_IDX_NAME].db,
                                               idxname,
                                               (reindex
-                                               || 
+                                               ||
                                                ((CNID_VERSION == CNID_VERSION_1) && (version == CNID_VERSION_0)))
                                               ? DB_CREATE : 0)) != 0) {
         LOG(log_error, logtype_cnid, "Failed to associate name index: %s", db_strerror(ret));
@@ -749,7 +667,7 @@ int dbif_open(DBD *dbd, struct db_param *dbp, int reindex)
         LOG(log_error, logtype_cnid, "Error upgrading CNID database to version %d", CNID_VERSION);
         return -1;
     }
-    
+
     return 0;
 }
 
@@ -798,7 +716,7 @@ int dbif_close(DBD *dbd)
     return 0;
 }
 
-/* 
+/*
    In order to support silent database upgrades:
    destroy env at cnid_dbd shutdown.
  */
@@ -809,11 +727,6 @@ int dbif_env_remove(const char *path)
 
     LOG(log_debug, logtype_cnid, "Trying to remove BerkeleyDB environment");
 
-    if (get_lock(LOCK_EXCL, path) != LOCK_EXCL) {
-        LOG(log_debug, logtype_cnid, "CNID db \"%s\" in use, can't remove BerkeleyDB environment", path);
-        return 0;
-    }
-    
     if (NULL == (dbd = dbif_init(path, "cnid2.db")))
         return -1;
 
@@ -828,7 +741,7 @@ int dbif_env_remove(const char *path)
         return -1;
 
     /* Open environment with recovery */
-    if ((ret = dbd->db_env->open(dbd->db_env, 
+    if ((ret = dbd->db_env->open(dbd->db_env,
                                  dbd->db_envhome,
                                  DB_CREATE | DB_INIT_LOG | DB_INIT_MPOOL | DB_INIT_LOCK | DB_INIT_TXN | DB_RECOVER | DB_PRIVATE,
                                  0))) {
@@ -917,7 +830,7 @@ int dbif_put(DBD *dbd, const int dbi, DBT *key, DBT *val, u_int32_t flags)
                                      val,
                                      flags);
 
-    
+
     if (ret) {
         if ((flags & DB_NOOVERWRITE) && ret == DB_KEYEXIST) {
             return 1;
@@ -938,7 +851,7 @@ int dbif_del(DBD *dbd, const int dbi, DBT *key, u_int32_t flags)
     if (dbd->db_cur) {
         dbd->db_cur->close(dbd->db_cur);
         dbd->db_cur = NULL;
-    }    
+    }
 
     if (dbif_txn_begin(dbd) < 0) {
         LOG(log_error, logtype_cnid, "error deleting key/value from %s", dbd->db_table[dbi].name);
@@ -949,7 +862,7 @@ int dbif_del(DBD *dbd, const int dbi, DBT *key, u_int32_t flags)
                                      dbd->db_txn,
                                      key,
                                      flags);
-    
+
     if (ret == DB_NOTFOUND) {
         LOG(log_debug, logtype_cnid, "key not found");
         return 0;
@@ -1050,7 +963,7 @@ int dbif_txn_commit(DBD *dbd)
 
     ret = dbd->db_txn->commit(dbd->db_txn, 0);
     dbd->db_txn = NULL;
-    
+
     if (ret) {
         LOG(log_error, logtype_cnid, "error committing transaction: %s", db_strerror(ret));
         return -1;
@@ -1071,7 +984,7 @@ int dbif_txn_abort(DBD *dbd)
 
     ret = dbd->db_txn->abort(dbd->db_txn);
     dbd->db_txn = NULL;
-    
+
     if (ret) {
         LOG(log_error, logtype_cnid, "error aborting transaction: %s", db_strerror(ret));
         return -1;
@@ -1079,7 +992,7 @@ int dbif_txn_abort(DBD *dbd)
         return 0;
 }
 
-/* 
+/*
    ret = 1 -> commit txn if db_param.txn_frequency
    ret = 0 -> abort txn db_param.txn_frequency -> exit!
    anything else -> exit!
@@ -1218,9 +1131,9 @@ int dbif_dump(DBD *dbd, int dumpindexes)
             did = ntohl(did);
 
             count++;
-            printf("id: %10u, did: %10u, type: %s, dev: 0x%llx, ino: 0x%016llx, name: %s\n", 
+            printf("id: %10u, did: %10u, type: %s, dev: 0x%llx, ino: 0x%016llx, name: %s\n",
                    cnid, did, typestring[type],
-                   (long long unsigned int)dev, (long long unsigned int)ino, 
+                   (long long unsigned int)dev, (long long unsigned int)ino,
                    (char *)data.data + CNID_NAME_OFS);
 
         }
@@ -1251,7 +1164,7 @@ int dbif_dump(DBD *dbd, int dumpindexes)
             LOG(log_error, logtype_cnid, "Couldn't create cursor: %s", db_strerror(rc));
             return -1;
         }
-        
+
         cur->c_get(cur, &key, &data, DB_FIRST);
         while (rc == 0) {
             /* Parse and print data */
@@ -1268,8 +1181,8 @@ int dbif_dump(DBD *dbd, int dumpindexes)
                 /* ino */
                 memcpy(&ino, (char *)key.data + CNID_DEV_LEN, CNID_INO_LEN);
                 ino = ntoh64(ino);
-                
-                printf("id: %10u <== dev: 0x%llx, ino: 0x%016llx\n", 
+
+                printf("id: %10u <== dev: 0x%llx, ino: 0x%016llx\n",
                        cnid, (unsigned long long int)dev, (unsigned long long int)ino);
                 count++;
             }
@@ -1279,7 +1192,7 @@ int dbif_dump(DBD *dbd, int dumpindexes)
             LOG(log_error, logtype_cnid, "Error iterating over btree: %s", db_strerror(rc));
             return -1;
         }
-        
+
         rc = cur->c_close(cur);
         if (rc) {
             LOG(log_error, logtype_cnid, "Couldn't close cursor: %s", db_strerror(rc));
@@ -1296,7 +1209,7 @@ int dbif_dump(DBD *dbd, int dumpindexes)
             LOG(log_error, logtype_cnid, "Couldn't create cursor: %s", db_strerror(rc));
             return -1;
         }
-        
+
         cur->c_get(cur, &key, &data, DB_FIRST);
         while (rc == 0) {
             /* Parse and print data */
@@ -1320,7 +1233,7 @@ int dbif_dump(DBD *dbd, int dumpindexes)
             LOG(log_error, logtype_cnid, "Error iterating over btree: %s", db_strerror(rc));
             return -1;
         }
-        
+
         rc = cur->c_close(cur);
         if (rc) {
             LOG(log_error, logtype_cnid, "Couldn't close cursor: %s", db_strerror(rc));
@@ -1332,7 +1245,7 @@ int dbif_dump(DBD *dbd, int dumpindexes)
     return 0;
 }
 
-/* 
+/*
    Iterates over dbd, returning cnids.
    Uses in-value of cnid to seek to that cnid, then gets next and return that in cnid.
    If close=1, close cursor.
@@ -1384,7 +1297,7 @@ int dbif_idwalk(DBD *dbd, cnid_t *cnid, int close)
     if (dbd->db_cur) {
         dbd->db_cur->close(dbd->db_cur);
         dbd->db_cur = NULL;
-    }    
+    }
 
     return 0;
 }
