@@ -13,26 +13,26 @@ Netatalk 2001 (c)
 
 ========================================================================= */
 
-#include <stdio.h>
-#include <limits.h>
-#include <stdarg.h>
-#include <string.h>
-#include <stdlib.h>
-#include <syslog.h>
-#include <sys/types.h>
-#include <sys/stat.h>
-#include <fcntl.h>
-#include <sys/uio.h>
-#include <unistd.h>
-#include <sys/time.h>
-#include <time.h>
 #include <ctype.h>
 #include <errno.h>
+#include <fcntl.h>
+#include <limits.h>
+#include <stdarg.h>
 #include <stdbool.h>
+#include <stdio.h>
+#include <stdlib.h>
+#include <string.h>
+#include <sys/stat.h>
+#include <sys/time.h>
+#include <sys/types.h>
+#include <sys/uio.h>
+#include <syslog.h>
+#include <time.h>
+#include <unistd.h>
 
-#include <atalk/util.h>
 #include <atalk/logger.h>
 #include <atalk/unix.h>
+#include <atalk/util.h>
 
 #define COUNT_ARRAY(array) (sizeof((array))/sizeof((array)[0]))
 
@@ -50,7 +50,7 @@ Netatalk 2001 (c)
   "debug7",                       \
   "debug8",                       \
   "debug9",                       \
-  "maxdebug"}                        
+  "maxdebug"}
 
 /* these are the string identifiers corresponding to each logtype */
 #define LOGTYPE_STRING_IDENTIFIERS { \
@@ -77,8 +77,9 @@ log_config_t log_config = { 0 };
    0:               syslog ?
    -1:              logfiles fd
    log_none:        no logging by default
-   0:               Display options */
-#define DEFAULT_LOG_CONFIG {0, 0, -1, log_none, 0}
+   0:               Display options
+   false            timestamp_us */
+#define DEFAULT_LOG_CONFIG {0, 0, -1, log_none, 0, true}
 
 UAM_MODULE_EXPORT logtype_conf_t type_configs[logtype_end_of_list_marker] = {
     DEFAULT_LOG_CONFIG, /* logtype_default */
@@ -117,7 +118,8 @@ static int generate_message(char **message_details_buffer,
                             char *user_message,
                             int display_options,
                             enum loglevels loglevel,
-                            enum logtypes logtype)
+                            enum logtypes logtype,
+                            bool log_us_timestamp)
 {
     char *details;
     int    len;
@@ -126,7 +128,7 @@ static int generate_message(char **message_details_buffer,
     char buf[256];
 
     gettimeofday(&tv, NULL);
-    strftime(buf, sizeof(buf), "%b %d %H:%M:%S.", localtime(&tv.tv_sec));
+    strftime(buf, sizeof(buf), "%b %d %H:%M:%S", localtime(&tv.tv_sec));
     pid = getpid();
     const char *basename = strrchr(log_src_filename, '/');
     if (basename) {
@@ -135,18 +137,30 @@ static int generate_message(char **message_details_buffer,
         basename = log_src_filename;
     }
 
-
-    len = asprintf(&details,
-                   "%s%06u %s[%d] {%s:%d} (%s:%s): %s\n",
-                   buf,
-                   (int)tv.tv_usec,
-                   log_config.processname,
-                   pid,
-                   basename,
-                   log_src_linenumber,
-                   arr_loglevel_strings[loglevel],
-                   arr_logtype_strings[logtype],
-                   user_message);
+    if (log_us_timestamp) {
+        len = asprintf(&details,
+                    "%s.%06u %s[%d] {%s:%d} (%s:%s): %s\n",
+                    buf,
+                    (int)tv.tv_usec,
+                    log_config.processname,
+                    pid,
+                    basename,
+                    log_src_linenumber,
+                    arr_loglevel_strings[loglevel],
+                    arr_logtype_strings[logtype],
+                    user_message);
+    } else {
+        len = asprintf(&details,
+                    "%s %s[%d] {%s:%d} (%s:%s): %s\n",
+                    buf,
+                    log_config.processname,
+                    pid,
+                    basename,
+                    log_src_linenumber,
+                    arr_loglevel_strings[loglevel],
+                    arr_logtype_strings[logtype],
+                    user_message);
+    }
     if (len == -1) {
         *message_details_buffer = "";
         return -1;
@@ -197,7 +211,7 @@ static void log_init(void)
                  logfacility_daemon);
 }
 
-static void log_setup(const char *filename, enum loglevels loglevel, enum logtypes logtype)
+static void log_setup(const char *filename, enum loglevels loglevel, enum logtypes logtype, const bool log_us_timestamp)
 {
     if (loglevel == 0) {
         /* Disable */
@@ -290,20 +304,22 @@ static void log_setup(const char *filename, enum loglevels loglevel, enum logtyp
     if (logtype == logtype_default) {
         int typeiter = 0;
         while (typeiter != logtype_end_of_list_marker) {
-            if ( ! (type_configs[typeiter].set))
+            if ( ! (type_configs[typeiter].set)) {
                 type_configs[typeiter].level = loglevel;
+                type_configs[typeiter].timestamp_us = log_us_timestamp;
+            }
             typeiter++;
         }
     }
 
-    LOG(log_debug, logtype_logger, "Setup file logging: type: %s, level: %s, file: %s",
-        arr_logtype_strings[logtype], arr_loglevel_strings[loglevel], filename);
+    LOG(log_debug, logtype_logger, "Setup file logging: type: %s, level: %s, file: %s, timestamp_us: %d",
+        arr_logtype_strings[logtype], arr_loglevel_strings[loglevel], filename, log_us_timestamp);
 }
 
 /* Setup syslog logging */
 static void syslog_setup(int loglevel, enum logtypes logtype, int display_options, int facility)
 {
-    /* 
+    /*
      * FIXME:
      * this currently doesn't care if logtype is already logging to a file.
      * Fortunately currently there's no way a user could trigger this as afpd.conf
@@ -343,7 +359,7 @@ static void syslog_setup(int loglevel, enum logtypes logtype, int display_option
  *    else
  *       set to default logging
  */
-static void setuplog_internal(const char *loglevel, const char *logtype, const char *filename)
+static void setuplog_internal(const char *loglevel, const char *logtype, const char *filename, const bool log_us_timestamp)
 {
     unsigned int typenum, levelnum;
 
@@ -378,7 +394,7 @@ static void setuplog_internal(const char *loglevel, const char *logtype, const c
                      logfacility_daemon);
     } else {
         /* this must be a filelog */
-        log_setup(filename, levelnum, typenum);
+        log_setup(filename, levelnum, typenum, log_us_timestamp);
     }
 
     return;
@@ -401,7 +417,7 @@ void set_processname(const char *processname)
    So it must be shorter than MAXLOGSIZE
    ------------------------------------------------------------------------- */
 void make_log_entry(enum loglevels loglevel, enum logtypes logtype,
-                    const char *file, int line, char *message, ...)
+                    const char *file, const bool log_us_timestamp, int line, char *message, ...)
 {
     /* fn is not reentrant but is used in signal handler
      * with LOGGER it's a little late source name and line number
@@ -419,7 +435,7 @@ void make_log_entry(enum loglevels loglevel, enum logtypes logtype,
     if (!log_config.inited) {
       log_init();
     }
-    
+
     if (type_configs[logtype].syslog) {
         if (type_configs[logtype].level >= loglevel) {
             /* Initialise the Messages and send it to syslog */
@@ -468,7 +484,7 @@ void make_log_entry(enum loglevels loglevel, enum logtypes logtype,
                            type_configs[logtype].set ?
                            type_configs[logtype].display_options :
                            type_configs[logtype_default].display_options,
-                           loglevel, logtype);
+                           loglevel, logtype, log_us_timestamp);
     if (len == -1) {
         goto exit;
     }
@@ -480,7 +496,7 @@ exit:
     inlog = 0;
 }
 
-void setuplog(const char *logstr, const char *logfile)
+void setuplog(const char *logstr, const char *logfile, const bool log_us_timestamp)
 {
     char *ptr, *save;
     char *logtype, *loglevel;
@@ -507,7 +523,7 @@ void setuplog(const char *logstr, const char *logfile)
                 ptr++;
             c = *ptr;
             *ptr = 0;
-            setuplog_internal(loglevel, logtype, logfile);
+            setuplog_internal(loglevel, logtype, logfile, log_us_timestamp);
             *ptr = c;
         }
         ptr = strtok(NULL, ", ");
